/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.app.view.admin.AdminViewer");

dojo.require("dojo.i18n");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");

(function() { // module container

var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var ServiceResponseHandler 	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

/**
 * Widget to display a report.
 */
dojo.declare("com.ibm.team.reports.web.app.view.admin.AdminViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/admin/templates/AdminViewer.html"),
		//templateCssPath: dojo.moduleUrl("com.ibm.team.reports.web", "app/view/admin/templates/AdminViewer.css"),
				
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.app.internal", "ReportsMessages");
			this.updateAllSnapshotData = this.messages.updateAllSnapshotData;
			this.updateSnapshotData = this.messages.updateSnapshotData;
			this.listAllTables = this.messages.listAllTables;
			this.responseHandler = new ServiceResponseHandler(
				this,
				"_done",
				"_remoteError");
		},
		
		postCreate: function() {
			com.ibm.team.reports.web.app.view.admin.AdminViewer.superclass.postCreate.apply(this);
			this.connect(this.listTables, "onsubmit", this._listTablesHandler);
			this.connect(this.updateAll, "onsubmit", this._updateAllHandler);
			this.connect(this.update, "onsubmit", this._updateHandler);
			var handler = new ServiceResponseHandler(
				this,
				"_fillSnapshotList",
				"_remoteError");
			this._working();
			ReportsClient.getAllSnapshotNames(handler);
		},
		
		_listTablesHandler: function(e) {
			e.preventDefault();
			this._handleListTables();
		},
		
		_updateAllHandler: function(e) {
			e.preventDefault();
			this._handleUpdateAll();
		},
		
		_updateHandler: function(e) {
			e.preventDefault();
			this._handleUpdate();
		},
		
		_handleListTables: function() {
			var handler = new ServiceResponseHandler(
				this,
				"_fillTableList",
				"_remoteError");
			var string = this.snapshotList.options[this.snapshotList.selectedIndex].text;
			if (string.length == 0) {
				alert(this.messages.needSnapshotSelected);
				return;
			}
			this._working();
			ReportsClient.getSnapshotDescriptor(handler, {snapshotName: string});
		},
		
		_handleUpdateAll: function() {
			var handler = new ServiceResponseHandler(
				this,
				"_noop",
				"_noop");
			this._removeChildren(this.statusDiv);
			this._submitted();
			ReportsClient.postUpdateAllSnapshotData(handler);
		},
		
		_handleUpdate: function() {
			var handler = new ServiceResponseHandler(
				this,
				"_noop",
				"_noop");
			this._removeChildren(this.statusDiv);
			var string = this.snapshotList.options[this.snapshotList.selectedIndex].text;
			if (string.length == 0) {
				alert("Need a snapshot selected");
				return;
			}
			this._submitted();
			ReportsClient.postUpdateSnapshotData(handler, {snapshotName: string});
		},
		
		_working: function() {
			this._displayStatus(this.messages.requestSubmitted, this.messages.waitingForResponse);
		},
		
		_submitted: function() {
			this._displayStatus(this.messages.requestSubmitted, this.messages.checkActiveServices);
		},
		
		_noop: function() {
		},
		
		_done: function() {
			this._displayStatus(this.messages.responseReceived, this.messages.lastRequestSucceeded);
		},
		
		_remoteError: function() {
			this._displayStatus(this.messages.errorInResponse, this.messages.lastRequestFailed);
		},
		
		_displayStatus: function(line1, line2) {
			var text1 = document.createTextNode(line1);
			var br = document.createElement("br");
			var text2 = document.createTextNode(line2);
			this._removeChildren(this.statusDiv);
			this.statusDiv.appendChild(text1);
			this.statusDiv.appendChild(br);
			this.statusDiv.appendChild(text2);
		},
		
		_removeChildren: function(element) {
			while (element.firstChild) {
			    element.removeChild(element.firstChild);
			}
		},
		
		_fillSnapshotList : function(items) {
			if (items != null) {
				for (var i = 0; i < items.length; i++) {
					var item = items[i];
					if (item != null) {
						var option = document.createElement("option");
						option.text = item;
						try {
    						this.snapshotList.add(option, null);
    					} catch(e) {
    						this.snapshotList.add(option);
    					}
					}
				}
			}
			this._done();
		},
		
		_fillTableList : function(element) {
			this._removeChildren(this.tablesDiv);
			if (element != null) {
				var p = document.createElement("p");
				this.tablesDiv.appendChild(p);
				p.innerHTML = dojo.string.substitute(this.messages.schemaDatabaseName, [element.dbName]);
				p = document.createElement("p");
				this.tablesDiv.appendChild(p);
				p.innerHTML = dojo.string.substitute(this.messages.schemaVersion, [element.version]);
				var tables = element.tables;
				for (var i = 0; i < tables.length; i++) {
					var table = document.createElement("table");
					this.tablesDiv.appendChild(table);
					var br = document.createElement("br");
					this.tablesDiv.appendChild(br);
					var row = table.insertRow(0);
					row.className = "metaDataTable";
					var cell = row.insertCell(0);
					cell.innerHTML = this.messages.tableLogicalName;
					cell = row.insertCell(1);
					cell.innerHTML = tables[i].name;
					row = table.insertRow(1);
					row.className = "metaDataTable";
					cell = row.insertCell(0);
					cell.innerHTML = this.messages.tableDatabaseName;
					cell = row.insertCell(1);
					cell.innerHTML = tables[i].dbName;
					row = table.insertRow(2);
					cell = row.insertCell(0);
					row = table.insertRow(3);
					row.className = "metaDataHeading";
					cell = row.insertCell(0);
					cell.innerHTML = this.messages.fieldLogicalName;
					cell = row.insertCell(1);
					cell.innerHTML = this.messages.fieldDatabaseName;
					cell = row.insertCell(2);
					cell.innerHTML = this.messages.fieldLogicalType;
					cell = row.insertCell(3);
					cell.innerHTML = this.messages.fieldDatabaseType;
					var fields = tables[i].fields;
					var flag = true;
					for (var j = 0; j < fields.length; j++) {
						row = table.insertRow(j + 4);
						if (flag) {
							row.className = "metaDataField1";
						} else {
							row.className = "metaDataField2";
						}
						flag = !flag;
						cell = row.insertCell(0);
						cell.innerHTML = fields[j].name;
						cell = row.insertCell(1);
						cell.innerHTML = fields[j].dbName;
						cell = row.insertCell(2);
						cell.innerHTML = fields[j].kind;
						cell = row.insertCell(3);
						cell.innerHTML = fields[j].dbKind;
					}
				}
			}
			this._done();
		}
	}
);

})();