/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.client.internal.ViewletHelper");

dojo.require("com.ibm.team.dashboard.web.client.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.client.internal.ItemNameRegistry");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletMemento");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

(function() {

/*
 * Aliases
 */
var Constants = com.ibm.team.dashboard.web.client.internal.Constants;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ItemNameRegistry = com.ibm.team.dashboard.web.client.internal.ItemNameRegistry;
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ViewletMemento = com.ibm.team.dashboard.web.client.internal.ViewletMemento;

/*
 * Models a viewlet.
 */
dojo.declare("com.ibm.team.dashboard.web.client.internal.ViewletHelper",
	null, {
	
	/*
	 * Constructs a viewlet helper.
	 */
	constructor: function(/* object */ rawHelper, /* DashboardItem */ item) {
		this._definitionId = rawHelper.definitionId;
		this._icon = rawHelper.icon;
		this._id = rawHelper.id;
		if (rawHelper.memento) {
			new ViewletMemento(dojo.fromJson(rawHelper.memento), this);
		}
		this._preferences = new Object();
		if (rawHelper.preferences) {
			for (var i=0;i<rawHelper.preferences.length;i++) {
				var entry = rawHelper.preferences[i];
				this._preferences[entry.id] = entry.value;
			}
		}
		this._scope = rawHelper.scope;
		if (rawHelper.scopeItem && rawHelper.scopeItem.itemId) {
			this._scopeItem = {itemId: rawHelper.scopeItem.itemId, name: ItemNameRegistry.getName(rawHelper.scopeItem.itemId)};
		}
		if (rawHelper.scopeSubteams || rawHelper.scopeSubteams == false) {
			this._scopeSubteams = rawHelper.scopeSubteams;
			if (typeof this._scopeSubteams  == "string") {
				this._scopeSubteams = (this._scopeSubteams  == "true");
			}
		}
		this._title = rawHelper.title;
		this._trim = rawHelper.viewletTrim;
		this._isShowBackground = (rawHelper.showBackground == true);
		this._isUserTitle = (rawHelper.userTitle == true);
		this._item = item;
	},
	
	/*
	 * Returns the dashboard item that contains this viewlet.
	 */
	/* DashboardItem */ getDashboardItem: function() {
		return this._item;
	},
	
	/*
	 * Returns the viewlet's definition id.
	 */
	/* string */ getDefinitionId: function() {
		return this._definitionId;
	},
	
	/*
	 * Returns the viewlet's icon path.
	 */
	/* string */ getIcon: function() {
		return this._icon;
	},
	
	/*
	 * Returns the viewlet id.
	 */
	/* string */ getId: function() {
		return this._id;
	},
	
	/*
	 * Returns the viewlet's memento, containing its persisted state.
	 */
	/* ViewletMemento */ getMemento: function() {
		return this._memento;
	},
	
	/*
	 * Returns the viewlet's preferences as an associative array.
	 */
	/* object */ getPreferences: function() {
		return this._preferences;
	},
	
	/*
	 * Returns the viewlet's scope. Must be one of the static
	 * constants DashboardConstants.SCOPE_CONTRIBUTOR, DashboardConstants.SCOPE_TEAM_AREA,
	 * DashboardConstants.SCOPE_PROJECT_AREA.
	 */
	/* number */ getScope: function() {
		switch(this._scope) {
			case "CONTRIBUTOR":
				return DashboardConstants.SCOPE_CONTRIBUTOR;
			case "TEAM_AREA":
				return DashboardConstants.SCOPE_TEAM_AREA;
			case "PROJECT_AREA":
				return DashboardConstants.SCOPE_PROJECT_AREA;
			default:
				return this._scope;
		}
	},
	
	/*
	 * Returns the viewlet's scope item. e.g. if the scope is a
	 * contributor, the scope item is the selected contributor.
	 */
	/* object */ getScopeItem: function() {
		return this._scopeItem;
	},
	
	/*
	 * Returns a boolean indicating if the scope item's subteams
	 * should be included. Only applicible if the viewlet is scoped
	 * to a team or project and the viewlet definition specifies the field as
	 * valid (include-subteams is "yes" or "yes-and-check").
	 */
	/* boolean */ getScopeSubteams: function() {
		return this._scopeSubteams;
	},
	
	/*
	 * Returns the viewlet's title.
	 */
	/* string */ getTitle: function() {
		return this._title;
	},

	/*
	 * Returns the viewlet's trim.
	 */
	/* number */ getTrim: function() {
		return this._trim;
	},
	
	/*
	 * Returns whether or not the viewlet is showing a background color.
	 */
	/* boolean */ isShowBackground: function() {
		return this._isShowBackground;
	},
	
	/*
	 * Returns whether or not the user has overriden the dynamic title.
	 */
	/* boolean */ isUserTitle: function() {
		return this._isUserTitle;
	},

	/*
	 * Sets the viewlet's user title state. True means the user
	 * has overriden the dynamic title.
	 */
	setUserTitle: function(/* boolean */ isUserTitle) {
		this._isUserTitle = isUserTitle;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_USER_TITLE, viewletId:this._id, isUserTitle:isUserTitle});
	},

	/*
	 * Sets the viewlet's preferences.
	 */
	setPreferences: function(/* object */ preferences) {
		dojo.mixin(this._preferences, preferences);
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_PREFERENCES, viewletId:this._id, preferences:preferences});
	},
	
	/*
	 * Sets the viewlet's scope. A value of -1 indicates that the scope should be cleared
	 * so the viewlet will use the dashboard's scope.
	 */
	setScope: function(/* number */ scope) {
		if (scope == -1)
			delete this._scope;
		else
			this._scope = scope;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_SCOPE, viewletId:this._id, scope:scope});
	},
	
	/*
	 * Sets the viewlet's scope item.
	 */
	setScopeItem: function(/* string */ scopeItemId) {
		if (scopeItemId == -1)
			delete this._scopeItem;
		else
			this._scopeItem = {itemId: scopeItemId, name: ItemNameRegistry.getName(scopeItemId)};
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_SCOPE_ITEM, viewletId:this._id, scopeItemId:scopeItemId});
	},
	
	/*
	 * Sets a boolean indicating if subteams of the selected team scope
	 * should be used.
	 */
	setScopeSubteams: function(/* boolean */ scopeSubteams) {
		this._scopeSubteams = scopeSubteams;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_SCOPE_SUBTEAMS, viewletId:this._id, scopeSubteams:scopeSubteams});
	},
	
	/*
	 * Sets the viewlet's title.
	 */
	setTitle: function(/* string */ title) {
		this._title = title;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_TITLE, viewletId:this._id, title:title});
	},
	
	/*
	 * Sets whether or not the viewlet should is showing a background color.
	 */
	setShowBackground: function(/* boolean */ isShowBackground) {
		this._isShowBackground = isShowBackground;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_SHOW_BACKGROUND, viewletId:this._id, isShowBackground:isShowBackground});
	},

	/*
	 * Sets the viewlet's trim.
	 */
	setTrim: function(/* number */ trim) {
		this._trim = trim;
		this._item.addModification({type:Constants.MODIFICATION_TYPE_SET_VIEWLET_TRIM, viewletId:this._id, trim:trim});
	}
});

})();