/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.Viewlet");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("com.ibm.team.dashboard.web.client.internal.ContributorMemento");
dojo.require("com.ibm.team.dashboard.web.client.internal.ViewletMemento");
dojo.require("com.ibm.team.dashboard.web.ui.ViewletSite");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.ui.IMemento");
dojo.require("com.ibm.team.dashboard.web.ui.internal.Constants");
dojo.require("com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker");
dojo.require("com.ibm.team.repository.web.client.session.Session");

(function() {

var IMemento = com.ibm.team.dashboard.web.ui.IMemento;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var Constants = com.ibm.team.dashboard.web.ui.internal.Constants;
var ContributorMemento = com.ibm.team.dashboard.web.client.internal.ContributorMemento;
var ServiceTracker = com.ibm.team.dashboard.web.ui.internal.util.ServiceTracker;
var ViewletMemento = com.ibm.team.dashboard.web.client.internal.ViewletMemento;
var ViewletSite = com.ibm.team.dashboard.web.ui.ViewletSite;

var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

/*
 * An abstract viewlet class which all viewlets must subclass.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.Viewlet",
	[dijit._Widget, dijit._Templated], {

	/*
	 * Base class constructor; do not call or override in viewlets.
	 */
	constructor: function(args) {
		this._definition = args.definition;
		this._helper = args.helper;
		this._site = new ViewletSite({
			container: args.container,
			viewletHelper: this._helper
		});
		this._zoomed = false;
		this._editing = false;
	},
	
	destroy: function() {
		this._isDestroyed = true;
		// abort any remaining requests
		if (this._serviceTracker) {
			var requests = this._serviceTracker.getActiveRequests();
			for (var i=0;i<requests.length;i++) {
				requests[i].abort();
			}
		}
		this.inherited(arguments);
	},

	/*
	 * Called by the framework after the viewlet is instantiated, but before the
	 * first refresh. Viewlets that need to initialize themselves shoudl override.
	 */
	init: function() {
	},
	
	/*
	 * Returns whether or not this viewlet has been destroyed.
	 */
	isDestroyed: function() {
		return this._isDestroyed ? true : false;
	},
	
	/*
	 * Returns the ViewletSite for this viewlet. The ViewletSite is used as a facade
	 * to the ViewletContainer allowing viewlets to perform various actions at the
	 * framework level.
	 */
	/* ViewletSite */ getSite: function() {
		return this._site;
	},
	
	/*
	 * Returns a per-user memento for persisting viewlet state info that can be
	 * different for each user, e.g. appearance.
	 */
	/* IMemento */ getContributorState: function() {
		if (!this._contributorMemento) {
			this._contributorMemento = new ContributorMemento(this._helper.getDashboardItem().getId(), this._helper.getId());
		}
		return this._contributorMemento;
	},
	
	/*
	 * Returns the viewlet's ServiceTracker, which subclasses should
	 * register every service call with. Any remaining requests will
	 * automatically be aborted on viewlet destroy.
	 */
	/* ServiceTracker */ getServiceTracker: function() {
		if (!this._serviceTracker) {
			this._serviceTracker = new ServiceTracker();
		}
		return this._serviceTracker;
	},
	
	/*
	 * Returns a memento for persisting viewlet state info that is the same for
	 * all users, e.g. viewlet content.
	 */
	/* IMemento */ getViewletState: function() {
		if (!this._viewletMemento) {
			this._viewletMemento = this._helper.getMemento();
			if (!this._viewletMemento) {
				this._viewletMemento = new ViewletMemento(this.getDefaultState(), this._helper);
			}
		}
		return this._viewletMemento;
	},
	
	/*
 	 * Public Method
 	 * 
 	 * Recieves notification when the setting areas is opened or closed.
 	 * 
 	 * @param boolean opened
 	 * @return <none>
 	 */		
	settingsAreaToggled: function(opened) {
		// NO-OP
		// Subclasses to override
	},	
	
	/*
	 * This function is always called before the Memento for the viewlet is saved.
	 * Viewlets that persist their state and need to know when the Memento is about
	 * to be updated (e.g. when save is pressed) should override this method.
	 */
	preSaveState: function() {
	},
	
	/*
 	 * Public Method
 	 * 
 	 * If the "Save" button is clicked in edit mode, this method is called to 
 	 * validate the edited content before leaving edit mode and saving the 
 	 * edited content.  
 	 * 
 	 * @param <none>
 	 * @return boolean
 	 * 		Returns "true" if the content is valid.  The viewlet leaves edit
 	 *      mode and saves the edited content.
 	 *      Otherwise, returns "false" if the content is not valid.  The 
 	 *      viewlet stays in edit mode and does not save the edited content.
 	 */		
	validateEditedContent: function() {
		return true;
	},		
	
	/*
	 * This function is used to generate the default state for a viewlet when
	 * one has not been previously saved. Viewlets should override to provide
	 * their default state object.
	 */
	/* object */ getDefaultState: function() {
		return new Object();
	},
	
	/*
	 * Returns the viewlet preference by the given id.
	 */
	/* string */ getPreference: function(/* string */ id) {
		var prefDefs = this._definition.preferenceDefinitions;
		if (prefDefs) {
			for (var i=0;i<prefDefs.length;i++) {
				var prefDef = prefDefs[i];
				if (prefDef.id == id) {
					// first check the viewlet helper
					var value = this._helper.getPreferences()[id];
					if (value || value == false) {
						return this._cast(value, prefDef.preferenceType);
					}
					// fall back to definition default
					if (prefDef.defaultValue) {
						return this._cast(prefDef.defaultValue, prefDef.preferenceType);
					}
					// no value found
					return null;
				}
			}
		}
		return null;
	},
	
	/*
	 * Returns the viewlet's scope. If the viewlet is set to automatic scope,
	 * then the dashboard's scope is returned.
	 */
	/* number */ getScope: function() {
		var scope = this._helper.getScope();
		if (scope == DashboardConstants.SCOPE_CONTRIBUTOR || scope == DashboardConstants.SCOPE_TEAM_AREA || scope == DashboardConstants.SCOPE_PROJECT_AREA)
			return scope;
		scope = this._helper.getDashboardItem().getScope();
		switch (this._definition.appliesTo) {
			case Constants.SettingsArea.SCOPE_APPLIES_TO_TEAMS:
				return DashboardConstants.SCOPE_TEAM_AREA;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_CONTRIBUTORS:
				return DashboardConstants.SCOPE_CONTRIBUTOR;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROJECTS:
				return DashboardConstants.SCOPE_PROJECT_AREA;
			case Constants.SettingsArea.SCOPE_APPLIES_TO_PROCESS:
				if (scope == DashboardConstants.SCOPE_CONTRIBUTOR) {
					return DashboardConstants.SCOPE_PROJECT_AREA;
				}
		}
		return scope;
	},
	
	/*
	 * Returns the viewlet's scope item. If the viewlet's scope item is set
	 * to automatic, then a default will be assigned as follows:
	 * if the scope is contributor, the authenticated user will be used
	 * if the scope is project or team, the dashboard's scope item
	 * will be used.
	 */
	/* object */ getScopeItem: function() {
		var scopeItem = this._helper.getScopeItem();
		if (scopeItem)
			return scopeItem;
		return this._helper.getDashboardItem().getDefaultItem(this.getScope());
	},
	
	/*
	 * Returns the scope setting that indicates if subteams should be considered
	 * part of the viewlets scope. Only applicible when the scope is set to a team
	 * area and only to viewlets who define include-subteams as "yes" or "yes-and-check"
	 * in their definition.
	 */
	/* boolean */ getScopeSubteams: function() {
		var value = this._helper.getScopeSubteams();
		if (!value && value != false)
			value = this._definition.scopeSubteams == "yes-and-check";
		return value;
	},
	
	/*
	 * Returns the URL to navigate to when the viewlet title link is
	 * clicked.
	 * 
	 * Viewlet that have title-as-hyperlink set to true should override
	 * this method to return their specific URLs.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return null;
	},

	/*
 	 * Public Method
 	 * 
 	 * Refreshes the viewlet content.
 	 * 
 	 * @param <none>
 	 * @return <none>
 	 */	
	refresh: function() {
		// NO-OP
		// Viewlet subclass to override
	},
	
	/*
	 * Repopulates the viewlet. New content should not be retrieved from the server.
	 * Called when presentation of the viewlet is changed in some way, but new information
	 * is not required.
	 * 
	 * Viewlets should override.
	 */
	update: function() {
	},
	
	/*
	 * Called by the platform when the zoomed state of the viewlet
	 * changes. This only applies to viewlets declared as zoomable.
	 * Viewlets should implement update() and call isZoomed().
	 */
	setZoomed: function(/* boolean */ isZoomed) {
		this._zoomed = isZoomed;
		this.update();
	},
	
	/*
	 * Indicates whether or not this viewlet is zoomed. Should be called
	 * by zoomable viewlets when updating.
	 */
	/* boolean */ isZoomed: function() {
		return this._zoomed;
	},

	/*
 	 * Public Method
 	 * 
 	 * Called just before a viewlet zoom operation is invoked
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	preZoom: function() {
		// NO-OP
		// Viewlet subclass to override
	},

	/*
 	 * Public Method
 	 * 
 	 * Called right after a viewlet zoom operation is completed
 	 * 
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	postZoom: function() {
		// NO-OP
		// Viewlet subclass to override
	},
	
	/*
	 * To be overridden by viewlets that need to update their contents
	 * when dashboard enters edit mode.
	 */
	setDashboardEditing: function(editing) {

	},
	
	/*
	 * Called by the platform to inform the viewlet whether or not it is active.
	 * A viewlet is active only when it is visible in the user's browser meaning
	 * the dashboard is the current workbench page and the viewlet is on the currently
	 * displayed dashboard tab.
	 */
	setActive: function(/* boolean */ active) {
	},
	
	/*
	 * Called by the platform when the viewlet enters or leaves edit mode.
	 * This only applies to viewlets declared as editable. Viewlets should
	 * implement update() and call isEditing
	 */
	setEditing: function(/* boolean */ isEditing) {
		this._editing = isEditing;
		this.update();
	},
	
	/*
	 * Indicates whether or not this viewlet is in edit mode. Should be called
	 * by editable viewlets when updating.
	 */
	/* boolean */ isEditing: function() {
		return this._editing;
	},
	
	/*
	 * Called by the platform to determine if the zoom button should be created.
	 * Applicable to zoomable viewlets only. This method is used to determine if
	 * the current content of the viewlet requires zooming. Default implementation
	 * always returns true.
	 * 
	 * May be overriden by zoomable viewlets.
	 */
	/* boolean */ hasZoomableContent: function() {
		return true;
	},
	
	/*
	 * Called by the platform when the list for a dynamic preference is opened.
	 * This method should be overriden by any viewlet that defines dynamic preferences.
	 * The definition id of the preference will be passed in. The result should
	 * be:
	 * 
	 * 1) an object containing a property named 'options'. This property will
	 * contain an array of objects each of which contains a 'label' and a 'value'.
	 * (e.g. "{options: [{label: 'First Option', value: 'option1'}, {label: 'Second
	 * Option', value: 'option2'}]}" ).
	 * 
	 * 2) an object containing a property named 'groups'. This property will
	 * contain an array of objects each of which contains a 'name' and an 'options' array
	 * as described in (1).
	 * 
	 * 3) null to indicate that the preferences are not ready yet. This will display
	 * a message indicating that the preferences are being loaded and will trigger a
	 * polling loop to keep calling this function until preference options are returned.
	 * 
	 * In cases (1) and (2) above, a 'computedDefault' property can be added to the object.
	 * This will cause the settings area to prime the combobox accordingly.
	 * (e.g. "{options: [{label: 'First Option', value: 'option1'}, {label: 'Second
	 * Option', value: 'option2'}], computedDefault: 'option2'}" ).
	 * 
	 * Since the preference options may need to be computed based on dirty (unsaved)
	 * settings, the second parameter will provide a working copy of the viewlet's
	 * settings. It will contain 2 to 5 properties:
	 * 
	 * title: the display title of the viewlet.
	 * scope (where applicable): an integer representing the scope of the viewlet; if the viewlet is auto
	 *     scoped, this value will be resolved to the automatic scope, so it can be used
	 *     unconditionally. Only applicable to scopable viewlets.
	 * scopeItemId (where applicable): a string representing the itemId of the current scopeItem; if the viewlet
	 *     is set to auto scope item, this value will be resolved to the automatic scope,
	 *     so it can be used unconditionally. Only applicable to scopable viewlets.
	 * scopeSubteams (where applicable): a boolean indicating whether or not subteam scoping
	 *     is enabled. Only applicable when team scope is selected and the viewlet declares
	 *     itself as sensative to this option.
	 * preferences: an associative array (by preference id) of the viewlet's preferences
	 * 
	 * Note, any preferences which are in the process of loading will not be included in
	 * currentSettings.preferences. If a preference depends on another preference that is
	 * still loading it should return null to indicate that it is loading as well.
	 */
	getPreferenceOptions: function(/* string */ preferenceId, /* object */ currentSettings) {
	},
	
	/*
	 * Called by the framework to get an implementation of CustomPreferenceProvider to be used
	 * for the specified custom preference.
	 */
	getPreferenceProvider: function(/* string */ preferenceId) {
		return null;
	},
	
	/*
	 * Called by the platform immediately after the viewlet's settings are saved. This method
	 * should be overriden by any viewlet that does not wish to refresh on all setting changes
	 * or by viewlets that would like to do something more on setting changes.
	 * 
	 * The newSettings parameter contains the new settings values that are about to be applied
	 * and the oldSettings parameter contains the settings values that existed before the save.
	 * It is up to the viewlet to compare these values and react accordingly. The parameters
	 * will each be an object with 2 to 5 properties as follows:
	 * 
	 * title: the display title of the viewlet.
	 * scope (where applicable): an integer representing the scope of the viewlet; if the viewlet is auto
	 *     scoped, this value will be resolved to the automatic scope, so it can be used
	 *     unconditionally. Only applicable to scopable viewlets.
	 * scopeItemId (where applicable): a string representing the itemId of the current scopeItem; if the viewlet
	 *     is set to auto scope item, this value will be resolved to the automatic scope,
	 *     so it can be used unconditionally.  Only applicable to scopable viewlets.
	 * scopeSubteams (where applicable): a boolean indicating whether or not subteam scoping
	 *     is enabled. Only applicable when team scope is selected and the viewlet declares
	 *     itself as sensative to this option.
	 * preferences: an associative array (by preference id) of the viewlet's preferences
	 */
	settingsChanged: function(/* object */ oldSettings, /* object */ newSettings) {
		this.getSite().refresh();
	},
	
	/*
	 * Called by the platform when any setting is changed inside the open settings area.
	 * This function should return an array of strings representing the preference ids of
	 * computed list preferences that depends on indicated setting.
	 * 
	 * For a viewlet title change the settingId will be "title", for a scope change it
	 * will be "scope" and for a scopeItem change it will be "scopeItem". Note that a
	 * scope change will cause the scopeItem to change as well, but only one call will be
	 * made to settingChanging and the parameter will be "scope".
	 */
	/* string[] */ settingChanging: function(/* string */ settingId) {
		return new Array();
	},
	
	/*
	 * Returns a properly typed primitive according to the type specified.
	 * E.g. if you pass in a string "true" and the preferenceType is "BOOLEAN",
	 * it will return a boolean with the value true.
	 */
	/* any */ _cast: function(/* string */ value, /* string */ preferenceType) {
		if (preferenceType == "INTEGER") {
			var parsedValue = parseInt(value);
			if (isNaN(parsedValue)) {
				return null;
			}
			return parsedValue;
		}
		else if (preferenceType == "BOOLEAN") {
			if (value == true || value == false)
				return value;
			return value == "true" ? true: false;
		}
		return value;
	}
});

})();
