/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas");

dojo.require("dojo.i18n");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.process.web.client.projectarea");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var getCurrentProjectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea;
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.AllTeamAreas",
	[dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/AllTeamAreas.html"),
	
	constructor: function() {
		messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this.allTeamsProj = messages.allTeamsProj;
	},
	postCreate: function() {
		this._handleBlur();
		this._filterBox.disabled = true;
		this._projectLink.innerHTML = HTMLUtility.escapeHTML(getCurrentProjectArea().name);
		var serviceObject = {
			self: this,
			success: function(result) {
				if (result && result.children && result.children.length > 0) {
					this.self._areas = result.children;
				}
				this.self._populate();
			},
			failure: function(error) {
				dojo.publish(com.ibm.team.dashboard.web.ui.internal.Constants.serverError, [error]);
			}
		};
		var srh = new ServiceResponseHandler(serviceObject, "success", "failure");
		ProcessClient.getProjectHierarchy({uuid: getCurrentProjectArea().itemId}, srh);
	},
	
	_populate: function() {
		if (!this._areas) {
			this._createRow(messages.allTeamsNone);
		} else {
			this._sortAreas(this._areas);
			this._parseAreas(this._areas, 0);
		}
		this._filterBox.disabled = false;
	},
	
	_parseAreas: function(areas, indent, pathRoot) {
		for (var i = 0; i < areas.length; i++) {
			var area = areas[i];
			var thisPath = (pathRoot ? pathRoot + "/" : "") + encodeURIComponent(area.name);
			area.node = this._createRow(area.name, "#action=" + DashboardConstants.ACTION_VIEW_DASHBOARD + "&" + DashboardConstants.URL_PARAM_TEAM + "=" + thisPath, indent);
			area.name = area.name.toLowerCase();
			if (area.children)
				this._parseAreas(area.children, indent+1, thisPath);
		}
	},
	
	_createRow: function(text, href, indent) {
		var result = document.createElement("div");
		result.className = "resultRow";
		var resultHtml = [];
		if (indent) {
			result.style.paddingLeft = (1 + (indent * 2)) + "em";
		} else {
			result.style.paddingLeft = "1em";
		}
		if (href) {
			resultHtml.push("<a href='");
			resultHtml.push(href);
			resultHtml.push("'>");
		}
		resultHtml.push(HTMLUtility.escapeHTML(text));
		if (href) {
			resultHtml.push("</a>");
		}
		result.innerHTML = resultHtml.join("");
		this._resultsArea.appendChild(result);
		return result;
	},
	
	_doFilter: function(event) {
		if (this._currentFilter && this._currentFilter == this._filterBox.value.toLowerCase())
			return;
		this._currentFilter = this._filterBox.value.toLowerCase();
		if (this._areas) {
			if (this._currentFilter.length == 0) 
				this._clearFilter(this._areas);
			else {
				var hits = this._filterAreas(this._areas);
				if (!hits) {
					if (!this._noHitsRow) {
						this._noHitsRow = this._createRow(messages.allTeamsNoneMatch);
					}
					this._noHitsRow.style.display = "block";
				} else if (this._noHitsRow)
					this._noHitsRow.style.display = "none";
			}
		}
	},
	
	_filterAreas: function(areas) {
		var result = false;
		for (var i = 0; i < areas.length; i++) {
			var childHits = false;
			if (areas[i].children && areas[i].children.length > 0)
				childHits = this._filterAreas(areas[i].children);
			if (childHits || areas[i].name.indexOf(this._currentFilter) != -1) {
				areas[i].node.style.display = "block";
				result = true;
			} else {
				areas[i].node.style.display = "none";
			}
		}
		return result;
	},
	
	_clearFilter: function(areas) {
		if (this._noHitsRow)
			this._noHitsRow.style.display = "none";
		for (var i = 0; i < areas.length; i++) {
			areas[i].node.style.display = "block";
			if (areas[i].children && areas[i].children.length > 0)
				this._clearFilter(areas[i].children);
		}
	},
	
	_handleClearFilter: function(event) {
		event.preventDefault();
		this._filterBox.value = "";
		this._clearFilterLink.blur();
	},
	
	_handleFocus: function(event) {
		if (dojo.hasClass(this._filterBox, "helpMessage")) {
			this._filterBox.value = "";
			dojo.removeClass(this._filterBox, "helpMessage");
			dojo.removeClass(this._clearFilterLink, "hidden");
		}
	},
	
	_handleBlur: function(event) {
		if (this._filterBox.value.length == 0) {
			// it is possible to clear and leave the box before the filter has a chance to run
			// ensure that it runs before we change the value of the box (call will be ignored if the filter is the same)
			this._doFilter();
			dojo.addClass(this._filterBox, "helpMessage");
			dojo.addClass(this._clearFilterLink, "hidden");
			this._filterBox.value = messages.filter;
		}
	},
	
	_sortAreas: function(areas) {
		areas.sort(function(a,b) {
			return a.name < b.name ? -1 : 1;
		});
		for (var i = 0; i < areas.length; i++) {
			if (areas[i].children && areas[i].children.length > 0) 
				this._sortAreas(areas[i].children);
		}
	}
	
});

})();
