/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.QuickSearch");

dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.require("com.ibm.team.dashboard.web.client.internal.DashboardClient");
dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");
dojo.require("com.ibm.team.dashboard.web.util.internal.HTMLUtility");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dijit._Templated");
dojo.require("dijit._Widget");
dojo.require("dijit.Menu");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");

(function() {

/*
 * Aliases
 */
var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;
var DashboardClient = com.ibm.team.dashboard.web.client.internal.DashboardClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var HTMLUtility = com.ibm.team.dashboard.web.util.internal.HTMLUtility;
var Util = com.ibm.team.dashboard.web.util.internal.Util;

/*
 * Constants
 */
var DELAY = 400;
var SCOPE_ALL = 0;
var SCOPE_TEAMS = 1;
var SCOPE_CONTRIBUTORS = 2;
var LABEL_SCOPE;
var LABEL_NO_RESULTS;
var CLASS_ICON = ["all-dashboards", "team-dashboards", "contributor-dashboards"];
var CLASS_MENU = ["menu-all", "menu-team", "menu-contributor"];

/*
 * Quick search in the left nav that searches as you type.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.QuickSearch",
	[dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/QuickSearch.html"),
	
	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		if (!LABEL_SCOPE) {
			LABEL_SCOPE = [
				this._messages.leftSearchScopeAll,
				this._messages.leftSearchScopeAllTeams,
				this._messages.leftSearchScopeAllProjs
			];
		}
		if (!LABEL_NO_RESULTS) {
			LABEL_NO_RESULTS = [
				this._messages.leftSearchNoDash,
				this._messages.leftSearchNoTeam,
				this._messages.leftSearchNoProj
			];
		}
		this._dashboardPage = args.dashboardPage;
		this._currentQueryId = 0;
		this._lastQueryIdReceived = -1;
	},
	
	postCreate: function() {
		this._searchText.value = this._messages.leftSearchDefaultText;
		this._setScope(SCOPE_ALL, false);
		this.connect(this._searchText, "onkeyup", this._onkeyup);
		this.connect(this._searchText, "onkeypress", this._onkeypress);
		this.connect(this._searchText, "onfocus", this._onfocus);
		this.connect(this._searchText, "onblur", this._onblur);
		this.connect(this._scopeMenu, "onclick", this._openMenu);
		this.connect(this._scopeMenu, "onkeypress",this._sniffKey);
	},

	_onkeyup: function(event) {
		var query = dojo.trim(this._searchText.value);		
		if (query != this._currentQuery) {
			this._currentQuery = query;
			if (!this._timeout) {
				// wait a bit then check
				this._timeout = setTimeout(dojo.hitch(this, this._update), 100);
			}
		}
	},

	_onkeypress: function(e) {
		if (e && e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) {
			if (e.keyCode == dojo.keys.DOWN_ARROW && this._results) {
				// down arrow from text field jumps to drop down
				this._results.focus();
			}
			if (e.keyCode == dojo.keys.ENTER && this._results) {
				// enter is equivalent to clicking on first matching result
				this._results.performDefault();
				this._searchText.blur();
			}
			dojo.stopEvent(e);
		}
		if (e && e.keyCode == dojo.keys.TAB && !e.shiftKey && this._isResultsOpen) {
			this._results.focus();
			dojo.stopEvent(e);
		}
	},
	
	_onfocus: function(e) {
		if (this._searchText.value == this._messages.leftSearchDefaultText) {
			this._searchText.value = "";
		}
		dojo.removeClass(this._searchText, "inactive");
		this._closeMenu();
	},

	_onblur: function(e) {
		// kill the periodic update
		if (this._timeout) {
			clearTimeout(this._timeout);
			delete this._timeout;
		}
		// show default help text if empty
		if (dojo.trim(this._searchText.value) == "") {
			dojo.addClass(this._searchText, "inactive");
			this._searchText.value = this._messages.leftSearchDefaultText;
		}
	},

	_openDashboard: function(/* SearchResultDTO */ result, /* number */ scope) {
		if (this._dashboardPage.navigateAwayEvent(false)) {
			location.href = Util.getDashboardHref(scope, true, result.id, result.id, result.id);
			this._searchText.value = result.name;
			this._closeResults();
		}
	},
	
	_search: function(/* string */ query) {
		// call search service
		var serviceObj = {
			_self: this,
			_query: query,
			success: function(/* ResponseDTO */responseDTO){
				if (responseDTO.status == "OK") {
					// ignore old responses when we already have a newer one
					var queryId = parseInt(responseDTO.data.queryId);
					if (queryId > this._self._lastQueryIdReceived) {
						this._self._lastQueryIdReceived = queryId;
						this._self._showResults(responseDTO.data, this._query);
					}
				}
				else {
					this._self._showError(responseDTO.msg);
				}
			},
			failure: function(/* Error */ error) {
				this._self._showError(error.message);
			}
		};
		var includeTeams = (this._scope != SCOPE_CONTRIBUTORS);
		var includeContributors = (this._scope != SCOPE_TEAMS);
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		DashboardClient.search(this._currentQuery, this._currentQueryId++, includeTeams, includeContributors, srh);
		
		// show loading if not yet open
		if (!this._isResultsOpen) {
			this._showResults({message: this._messages.loading});
		}
	},
	
	_showError: function(/* string */ errorMsg) {
		this._showResults({message: errorMsg ? dojo.string.substitute(this._messages.leftSearchError, [ errorMsg ]) : this._messages.leftSearchErrorContact});
	},
	
	_showResults: function(/* SearchResultsDTO */ resultsDTO, /* string */ query) {
		if (!this._results) {
			this._results = new com.ibm.team.dashboard.web.ui.internal.SearchResults({predecessor: this._searchText, successor: this.domNode.nextSibling, onselect: dojo.hitch(this, this._openDashboard)});
			this._resultsCloseOnBody = this.connect(this, "_onBlur", this._closeResults);
		}

		var pos = dojo.coords(this._searchWrapper);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		this._results.setContent(resultsDTO, query, pos.w, this._scope);
		
		// open if needed
		if (!this._isResultsOpen && this._currentQuery.length > 0) {
			/*
			 * Workaround to avoid jumping. Can't use node style because dijit uses it,
			 * so use a class instead.
			 */
			dojo.addClass(this._results.domNode, "com-ibm-team-dashboard-web-ui-internal-SearchResults-hidden");
			dijit.popup.open({ popup: this._results, parent: this, x: pos.x - (dojo.isIE ? 0 : 1), y: pos.y + pos.h - (dojo.isIE ? 4 : 5), onExecute: dojo.hitch(this,this._closeResults),orient: 'BL' });
			dojo.removeClass(this._results.domNode, "com-ibm-team-dashboard-web-ui-internal-SearchResults-hidden");
			this._isResultsOpen = true;
		}
	},
	
	_closeResults: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		dijit.popup.close(this._results);
		this._isResultsOpen = false;
	},

	_update: function() {
		if (this._currentQuery != this._lastQuery) {
			this._lastQuery = this._currentQuery;
			if (this._currentQuery.length > 0) {
				this._search(this._currentQuery);
			}
			else {
				this._closeResults();
			}
			this._timeout = setTimeout(dojo.hitch(this, this._update), DELAY);
		}
		else {
			delete this._timeout;
		}
	},

	destroy: function() {
		if (this._results) {
			this._results.destroyRecursive();
			delete this._results;
		}
		if (this._menu) {
			this._menu.destroyRecursive();
			delete this._menu;
		}
		this.inherited(arguments);
	},

	_sniffKey: function(e){
		if (e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) {
			this._openMenu();
			dojo.stopEvent(e);
		}
	},
	
	_openMenu: function(e) {
		if (e){
			dojo.stopEvent(e);
		}
		if (this._menu) {
			this._closeMenu();
			return;
		}
		
		var menu = new dijit.Menu();
		for (var i=0;i<LABEL_SCOPE.length;i++) {
			var label = LABEL_SCOPE[i];
			var iconClass = CLASS_ICON[i];
			if (i == this._scope) {
				iconClass += "-selected";
			}
			menu.addChild(new dijit.MenuItem({label: label, iconClass: iconClass, onClick: dojo.hitch(this, this._setScope, i, true)}));
		}
		
		var pos = dojo.coords(this._searchForm);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		
		dojo.addClass(this._scopeMenu, "ScopeMenu-selected");
		dijit.popup.open({ popup: menu, parent: this, x: pos.x + 5, y: pos.y + pos.h - 2, onExecute: dojo.hitch(this,this._closeMenu),orient: 'BL' });
		
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this,"_onBlur",this._closeMenu);
		this._menuKeyHandler = this.connect(menu.domNode,"onkeypress",this._onMenuKeyPress);
		this._menu = menu;
		menu.focus();
	},
	
	_closeMenu: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._menu) {
			dojo.removeClass(this._scopeMenu, "ScopeMenu-selected");
			this.disconnect(this._menuCloseOnBody);
			this.disconnect(this._menuKeyHandler);
			dijit.popup.close(this._menu);
			this._menu.destroyRecursive();
			this._menu = null;
		}
	},
	
	_onMenuKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			if (e.shiftKey) {
				// previous item is menu drop down
				this._scopeMenu.focus();
			}
			else {
				// next item is text field
				this._searchText.focus();
			}
		}
	},
	
	_setScope: function(/* number */ scope, /* boolean */ focusText) {
		this._scope = scope;
		this._scopeMenu.title = dojo.string.substitute(this._messages.leftSearchMenuTooltip, [ LABEL_SCOPE[scope] ]);
		for (var i=0;i<CLASS_MENU.length;i++) {
			if (i == this._scope) {
				dojo.addClass(this._scopeMenu, CLASS_MENU[i]);
			}
			else {
				dojo.removeClass(this._scopeMenu, CLASS_MENU[i]);
			}
		}
		if (focusText) {
			this._searchText.focus();
		}
	}
});

/*
 * The drop-down search result widget.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.SearchResults",
	[dijit._Widget, dijit._Templated], {
	
	templateString: "<div class=\"com-ibm-team-dashboard-web-ui-internal-SearchResults\"></div>",
	
	/*
	 * Creates a SearchResults widget.
	 * 
	 * @param predecessor the node that gets focus if you press up arrow on the first result
	 * @param successor the node that gets focus if you press tab
	 * @param onselect the callback for when a result is opened (via click/enter)
	 */
	constructor: function(args) {
		this._messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "DashboardMessages");
		this._predecessor = args.predecessor;
		this._successor = args.successor;
		this._onselect = args.onselect;
	},
	
	focus: function() {
		this._menu.focus();
	},
	
	performDefault: function() {
		if (this._defaultHandler) {
			this._defaultHandler();
		}
	},
	
	setContent: function(resultsDTO, query, width, scope) {
		delete this._defaultHandler;
		if (query) {
			this._query = query.toLowerCase();
		}
		this._results = resultsDTO;
		this._width = width;
		
		// create a new menu
		var menu = new com.ibm.team.dashboard.web.ui.internal.CustomMenu({predecessor: this._predecessor, successor: this._successor});
		var message = this._results.message;
		if (message) {
			this._addItem(menu, {name: message}, null, false, 0);
		}
		else {
			var teams = this._results.teams;
			var contributors = this._results.contributors;
			if ((teams && teams.length > 0) || (contributors && contributors.length > 0)) {
				if (teams) {
					for (var i = 0; i < teams.length; i++) {
						this._addItem(menu, teams[i], DashboardConstants.SCOPE_TEAM_AREA, true, 0);
					}
				}
				if (contributors) {
					var indent = 0;
					if (teams && teams.length > 0) {
						this._addItem(menu, {name: dojo.string.substitute(this._messages.leftSearchUsers, [ contributors.length ])}, null, false, 0);
						indent++;
					}
					for (var i = 0; i < contributors.length; i++) {
						this._addItem(menu, contributors[i], DashboardConstants.SCOPE_CONTRIBUTOR, true, indent);
					}
				}
			}
			else {
				this._addItem(menu, {name: LABEL_NO_RESULTS[scope]}, null, false, 0);
			}
		}

		// create container
		var content = document.createElement("div");
		content.className = "content";
		content.style.width = (this._width - 34) + "px";
		content.appendChild(menu.domNode);
		
		// swap in new container
		if (this._content) {
			this.domNode.replaceChild(content, this._content);
		}
		else {
			this.domNode.appendChild(content);
		}
		this._content = content;
		
		// destroy old menu
		if (this._menu) {
			this._menu.destroyRecursive();
		}
		this._menu = menu;
	},
	
	_addItem: function(menu, result, scope, enabled, indent) {
		var label = HTMLUtility.escapeHTML(result.name);
		if (enabled && this._query) {
			label = this._formatLabel(label);
		}
		var buf = ["<div class=\"entry\" style=\"padding-left: "];
		buf.push((indent*1.25));
		buf.push("em\">");
		buf.push(label);
		buf.push("</div>");
		if (enabled) {
			var handler = dojo.hitch(this, this._onselect, result, scope);
			var item = new dijit.MenuItem({label: buf.join(""), onClick: handler});
		}
		else {
			var item = new com.ibm.team.dashboard.web.ui.internal.DisabledMenuItem({label: buf.join("")});
		}
		if (label === result.name) {
			dojo.addClass(item.domNode, "context");
		}
		else if (!this._defaultHandler) {
			this._defaultHandler = handler;
			dojo.addClass(item.domNode, "default");
		}
		menu.addChild(item);
		
		// add children recursively
		var children = result.children;
		if (children) {
			var childIndent = indent + 1;
			for (var i=0;i<children.length;i++) {
				this._addItem(menu, children[i], scope, true, childIndent);
			}
		}
	},
	
	/*
	 * Bolds the first instance of query in the given label.
	 */
	_formatLabel: function(/* string */ label) {
		var index = label.toLowerCase().indexOf(this._query);
		if (index != -1) {
			return label.substring(0, index) + "<b>" + label.substring(index, index + this._query.length) + "</b>" + label.substring(index + this._query.length);
		}
		return label;
	}
});

/*
 * A specialized menu that handles up/down keys differently than normal.
 * If you press down arrow and you're on the last node, it stays instead of
 * wrapping. If you're on the first node and you press up arrow, it will leave
 * the menu and go to a designated predecessor node.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.CustomMenu",
	dijit.Menu, {
	
	/*
	 * Constructs the menu with the given predecessor node that will receive
	 * focus when you press up arrow on the first node.
	 */
	constructor: function(args) {
		this._predecessor = args.predecessor;
		this._successor = args.successor;
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this.connect(this, "onkeypress", this._onKeyPress);
	},
	
	/*
	 * Override to prevent wrapping on the last node.
	 */
	focusNext: function() {
		if (this.focusedChild) {
			var children = this.getChildren();
			if (this.focusedChild == children[children.length-1]) {
				// prevent wrapping
				return;
			}
		}
		this.inherited(arguments);
	},

	/*
	 * Override to jump to predecessor node if on first node.
	 */	
	focusPrev: function() {
		if (this.focusedChild) {
			var children = this.getChildren();
			if (this.focusedChild == children[0]) {
				// jump to the menu's predecessor
				this._onChildBlur(this.focusedChild);
				this.focusedChild = null;
				this._predecessor.focus();
				return;
			}
		}
		this.inherited(arguments);
	},
	
	onItemHover: function(/* MenuItem */ item) {
		dojo.addClass(item.domNode, "dijitMenuItemHover");
	},
	
	onItemUnhover: function(/* MenuItem */ item) {
		dojo.removeClass(item.domNode, "dijitMenuItemHover");
	},
	
	_onKeyPress: function(e) {
		if (e.keyCode == dojo.keys.TAB) {
			if (e.shiftKey) {
				this._predecessor.focus();
			}
			else {
				this._successor.focus();
			}
		}
	}
});

/*
 * A grayed and non-functional menu item. You can't click or highlight it, and
 * up/down arrow keys will skip it.
 */
dojo.declare("com.ibm.team.dashboard.web.ui.internal.DisabledMenuItem",
	dijit.MenuItem, {

	/*
	 * Override to disable the menu item.
	 */
	postCreate: function(){
		this.inherited(arguments);
		dojo.addClass(this.domNode, "disabled");
		dojo.setSelectable(this.domNode, false);
	},
	
	/*
	 * Never receives focus.
	 */
	isFocusable: function(){
		return false;
	}
});

})();