/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat");

dojo.require("dojo.date");
dojo.require("dojo.date.locale");
dojo.require("dojo.i18n");
dojo.require("dojo.string");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages");

(function() {

/*
 * Formats dates as a readable description compared to the current time,
 * e.g. "5 minutes ago", "yesterday".
 * 
 * Ported from java class com.ibm.team.jface.HistoryBin.
 */
com.ibm.team.dashboard.viewlets.web.ui.internal.utils.RelativeDateFormat = {
	
	_messages: dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "FeedMessages"),
	
	/**
	 * <p>
	 * Compares the given Date with the current Time and generates a Label
	 * describing the difference. The given Date can either be in the future or
	 * in the past. Some sample results are: "1 Minute ago", "3 Hours ago" or
	 * "In 5 Minutes".
	 * </p>
	 * 
	 * @param date The Date to compare with the current Time. Can either be a
	 * future date or a past date. The Date's difference in Time to the current
	 * one should not be greater of 1 Day.
	 * @return Returns a String that describes the difference in Time like "1
	 * Minute ago", "3 Hours ago" or "In 5 Minutes".
	 * @see formatDif() in case the Date's difference in Time to the current one
	 * is bigger than 1 Day.
	 */
	getReadableTimeDif: function(/* Date */ date) {
		if (!date)
			return "";

		var now= new Date().getTime();
		var then= date.getTime();
		var dif= now - then;

		var second= 1000;
		var minute= second * 60;
		var hour= minute * 60;
		var day= hour * 24;

		var minuteIntervals= [ 1, 2, 3, 5, 10, 15, 20, 30, 45, 60 ];
		var hourIntervals= [ 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24 ];

		/* Event is Now (dif < 60 Seconds) */
		if (dif > -minute && dif < minute)
			return this._messages.now;

		/* Event is in the Future */
		else if (dif < 0) {
			dif*= -1;

			/* Minutes */
			if (dif < minute * 2)
				return this._messages.inMin;

			for (var i= 2; i < minuteIntervals.length; i++) {
				if (dif < minuteIntervals[i] * minute)
					return dojo.string.substitute(this._messages.inMins, [ minuteIntervals[i-1] ]);
			}

			/* Hours */
			if (dif < hour * 2)
				return this._messages.inHour;

			for (var i= 1; i < hourIntervals.length; i++) {
				if (dif < hourIntervals[i] * hour)
					return dojo.string.substitute(this._messages.inHours, [ hourIntervals[i-1] ]);
			}

			if (dif < 2 * day) {
				return this._messages.inDay;
			}
			return this._messages.inSeveralDays;
		}

		/* Event is in the Past */
		else {

			/* Minutes */
			if (dif < minute * 2)
				return this._messages.pastMin;

			for (var i= 2; i < minuteIntervals.length; i++) {
				if (dif < minuteIntervals[i] * minute)
					return dojo.string.substitute(this._messages.pastMins, [ minuteIntervals[i-1] ]);
			}

			/* Hours */
			if (dif < hour * 2)
				return this._messages.pastHour;

			for (var i= 2; i < hourIntervals.length; i++) {
				if (dif < hourIntervals[i] * hour)
					return dojo.string.substitute(this._messages.pastHours, [ hourIntervals[i-1] ]);
			}

			if (dif < 2 * day) {
				return this._messages.pastDay;
			}
			return this._messages.pastSeveralDays;
		}
	},

	/**
	 * Formats the given <code>Date</code> into a <code>String</code>
	 * representing the difference in Time compared to the current Time. This
	 * Method can deal with Dates that are in the Future.<br>
	 * <br>
	 * Examples of calling this Method with <code>TRUE</code> supplied for
	 * <code>showAbsoluteDate</code> and <code>showRelativeDate</code>:
	 * <ul>
	 * <li>Now + 5 Seconds: <code>12:15 PM (In 1 minute)</code></li>
	 * <li>Now - 5 Minutes: <code>12:10 PM (10 minutes ago)</code></li>
	 * <li>Now + 5 Hours: <code>5:15 PM (In 5 hours)</code></li>
	 * <li>Now + 1 Day: <code>Sep 30 12:26 PM (Tomorrow)</code></li>
	 * <li>Now - 2 Days: <code>Sep 27 12:26 PM (2 days ago)</code></li>
	 * <li>Now + 1 Week: <code>Oct 06 12:26 PM (Next Week)</code></li>
	 * <li>Now + 3 Weeks: <code>Oct 20 12:26 PM</code></li>
	 * <li>Now - 1 Year: <code>Sep 29 2005 12:26 PM</code></li>
	 * </ul>
	 * <p>
	 * Examples for passing <code>FALSE</code> for
	 * <code>showAbsoluteDate</code>:
	 * <ul>
	 * <li>Now + 5 Seconds: <code>In 1 minute</code></li>
	 * <li>Now - 5 Minutes: <code>10 minutes ago</code></li>
	 * </ul>
	 * </p>
	 * <p>
	 * Note: This Method is <em>not</em> thread-safe.
	 * </p>
	 * 
	 * @param date The Date to compare to the current Time.
	 * @param showAbsoluteDate If <code>TRUE</code>, the absolute Date is
	 * added to the result.
	 * @param showRelativeDate If <code>TRUE</code>, the relative difference
	 * in Time to the current Time is added to the result.
	 * @return A <code>String</code> representing the difference in Time
	 * compared to the current Time.
	 */
	formatDif: function(/* Date */ date, /* boolean */ showAbsoluteDate, /* boolean */ showRelativeDate) {
		var str= "";

		if (!date)
			return "";

		var fgCalBase = new Date();
		var fgCalActual = date;
		
		var baseDay= this._getDayOfYear(fgCalBase);
		var actualDay= this._getDayOfYear(fgCalActual);

		var baseWeek= this._getWeekOfYear(fgCalBase);
		var actualWeek= this._getWeekOfYear(fgCalActual);

		var baseYear= fgCalBase.getFullYear();
		var actualYear= fgCalActual.getFullYear();
		
		/* Same Day */
		if (baseDay == actualDay && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + this.getReadableTimeDif(date) + (showAbsoluteDate ? ")" : "");
		}

		/* Yesterday */
		else if ((baseDay - actualDay) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + this._messages.yesterday + (showAbsoluteDate ? ")" : "");
		}

		/* Tomorrow */
		else if ((actualDay - baseDay) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + this._messages.tomorrow + (showAbsoluteDate ? ")" : "");
		}

		/* Same Week */
		else if (baseWeek == actualWeek && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate) {
				var dayStr;
				var dayDif= actualDay - baseDay;
				if (dayDif * dayDif == 1) 
					dayStr = (dayDif > 0) ? this._messages.inDay : this._messages.pastDay;
				else
					dayStr = (dayDif > 0) ? dojo.string.substitute(this._messages.inDays, [dayDif]) : dojo.string.substitute(this._messages.pastDays, [-dayDif]);

				str += (showAbsoluteDate ? "(" : "") + dayStr + (showAbsoluteDate ? ")" : "");
			}
		}

		/* Last Week */
		else if ((baseWeek - actualWeek) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + this._messages.lastWeek + (showAbsoluteDate ? ")" : "");
		}

		/* Next Week */
		else if ((actualWeek - baseWeek) == 1 && baseYear == actualYear) {
			if (showAbsoluteDate)
				str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" }) + " " + dojo.date.locale.format(date, { selector: "time", formatLength: "short" });

			if (showAbsoluteDate && showRelativeDate)
				str += " ";

			if (showRelativeDate)
				str += (showAbsoluteDate ? "(" : "") + this._messages.nextWeek + (showAbsoluteDate ? ")" : "");
		}

		/* Everything else */
		else {
			str += dojo.date.locale.format(date, { selector: "date", formatLength: "medium" });
			if (showAbsoluteDate) {
				str += " " + dojo.date.locale.format(date, {selector: "time", formatLength: "short"});
			}
		}

		return str;
	},
	
	formatTime: function(/* int */ millis) {
		var timeLeft = Math.round(millis / 1000);
		var seconds = timeLeft % 60;
		timeLeft = Math.floor(timeLeft / 60);
		if (timeLeft > 0) {
			var minutes = timeLeft % 60;
			timeLeft = Math.floor(timeLeft / 60);
			if (timeLeft > 0) {
				var hours = timeLeft % 60;
				timeLeft = Math.floor(timeLeft / 60);
				if (timeLeft > 0) {
					var days = Math.floor(timeLeft / 24);
				}
			}
		}
		var result = this._messages.lessThanSec;
		if (seconds || minutes || hours || days) {
			if (days)
				result = days == 1 ? this._messages.day : dojo.string.substitute(this._messages.days, [ days ]);
			if (hours)
				result = (days ? result + ", " : "") + (hours == 1 ? this._messages.hour : dojo.string.substitute(this._messages.hours, [ hours ]));
			if (minutes)
				result = (days || hours ? result + ", " : "") + (minutes == 1 ? this._messages.min : dojo.string.substitute(this._messages.mins, [ minutes ]));
			if (seconds)
				result = (days || hours || minutes ? result + ", " : "") + (seconds == 1 ? this._messages.sec : dojo.string.substitute(this._messages.secs, [ seconds ]));
		}
		return result;
	},
	
	_getDayOfYear: function(/* Date */ date){
		return dojo.date.difference(new Date(date.getFullYear(), 0, 1), new Date(date.getFullYear(), date.getMonth(), date.getDate())) + 1;
	},

	_getWeekOfYear: function(/* Date */ date){
		var firstDayOfYear = new Date(date.getFullYear(), 0, 1).getDay();
		var adj = (firstDayOfYear + 7) % 7;
		var week = Math.floor((this._getDayOfYear(date) + adj - 1) / 7);
		if(firstDayOfYear == 0){ week++; }
		return week;
	}
};

})();
