/**
 * @author hyassine
 */

dojo.provide("com.ibm.team.scm.web.ui.internal.view.browse.StreamWorkspaceView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojo.i18n");
dojo.require("dijit._base.scroll");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.scm.web.client.util");
dojo.require("com.ibm.team.scm.web.client.internal.ScmClient");

dojo.requireLocalization("com.ibm.team.scm.web.internal", "Messages");

(function() { // module container

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ScmUtil					= com.ibm.team.scm.web.client.util;
var ScmClient				= com.ibm.team.scm.web.client.internal.ScmClient;

dojo.declare(
	"com.ibm.team.scm.web.ui.internal.view.browse.StreamWorkspaceView",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.scm.web", "ui/internal/view/browse/templates/StreamWorkspaceView.html"),
	
		controller: null,
		
		PAGE_SIZE: 25,				//Default component history paging size
		currentConfigPageSize: 25,	//Current set component history paging size
		
		//Flags used to check for change set changes
		NONE: 0x00,
		ADD: 0x01,
		MODIFY: 0x02,
		RENAME: 0x04,
		REPARENT: 0x08,
		DELETE: 0x10,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.scm.web.internal", "Messages");
		},
		
		postCreate: function() {
		},
		
		/**
		 * Sets the content of the main page view to either display components, files/folders, file content,
		 * component history, item history, active team changes, or change sets.
		 * 
		 * @param {Object} args
		 * @param {Object} subElements
		 */
		setContent: function(args, subElements) {
			//Clear current page view.
			dojox.data.dom.removeChildren(this.componentList);
			dojox.data.dom.removeChildren(this.currentPath);
			dojox.data.dom.removeChildren(this.tableHeading);
			dojox.data.dom.removeChildren(this.changeOverview);
			dojox.data.dom.removeChildren(this.fileView);
			dojox.data.dom.removeChildren(this.flowTargets);
			
			if(args.isMainPage) {
				this._setMainPage();
			}
			else if(args.isComponentHistory) {
				this._displayHistory(subElements, "component");
			}
			else if(args.isItemHistory) {
				this._displayHistory(subElements, "item");
			}
			else if(args.isTeamChanges) {
				this._displayTeamChanges(subElements);
			}
			else if(args.isChangeSetFromHistory) {
				this._displayChangeSet(subElements, 1);
			}
			else if(args.isChangeSetFromActivity) {
				this._displayChangeSet(subElements, 0);
			}
			else if(args.isChangeSetItem) {
				this._displayChangeSetItem(subElements);
			}
			else if(args.isSnapshots) {
				this._displaySnapshots(subElements);
			}
			else {
				//Stream or workspace selected
			 	if(args.isStream || args.isWorkspace) {
					this._displayWorkspaceComponents(subElements.components, subElements.workspace.itemId);
					if(args.isStream) {
						this._buildWorkspaceNavigationPath(subElements.workspace.name, subElements.workspace.itemId, "stream");
					}
					else if(args.isWorkspace) {
						this._buildWorkspaceNavigationPath(subElements.workspace.name, subElements.workspace.itemId, "workspace");					
					}
					this._addWorkspaceInfo(subElements);
				}
				//Component, folder, or file selected
				else if(args.isOther || args.isComponent) {
					//Display items under component/folder
					if(subElements.children) {
						this._buildSubElementNavigationPath(subElements.workspace, subElements.component, subElements.path);
						this._addHistoryLink(subElements);
						this._displaySubElements(subElements);
					}
					else {
						//Display an empty directory
						if(subElements.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder") {		
							this._buildSubElementNavigationPath(subElements.workspace, subElements.component, subElements.path);
							this._addHistoryLink(subElements);
							this._displayEmptyDirectory(subElements.path);
						}
						//Display the file content
						else if(subElements.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.filesystem.FileItem") {
							
							this._buildSubElementNavigationPath(subElements.workspace, subElements.component, subElements.path, "true");
							
							this._addHistoryAndDownloadLinks(subElements);
							
							//Build the file content area
							var tableDiv = document.createElement("div");
							tableDiv.className = "table-div";
							var table = document.createElement("table");
							table.className = "fileContent";
							var thead = document.createElement("thead");
							var tr = document.createElement("tr");
							tr.className = "item-name";
							var th = document.createElement("th");
							th.className = "item-name";
							var text = document.createTextNode(subElements.versionable.name);
							th.appendChild(text);
							tr.appendChild(th);
							thead.appendChild(tr);
							
							table.appendChild(thead);
							
							var tbody = document.createElement("tbody");
							var tr = document.createElement("tr");
							var td = document.createElement("td");

							if (this._hasTextContentType(subElements.versionable)) {
								//An inner table is used since COLSPAN is not properly supported in IE
								var innerTable = document.createElement("table");
								innerTable.className = "file";
								var thead = document.createElement("thead");
								var innerTr = document.createElement("tr");
								innerTr.className = "line-heading";
								var th = document.createElement("th");
								th.className = "line-heading";
								var text = document.createTextNode(this.messages.StreamWorkspaceView_24);
								th.appendChild(text);
								innerTr.appendChild(th);
								var th = document.createElement("th");
								th.className = "content-heading";
								innerTr.appendChild(th);
								thead.appendChild(innerTr);
								innerTable.appendChild(thead);
								
								var innerTbody = document.createElement("tbody");
								
								var headers = {};
								headers["accept-charset"] = "utf-8";
								
								var self = this;
								var xhr = dojo.xhrGet({
									url:"../../service/com.ibm.team.scm.common.internal.rest.IScmWebContentService/" + subElements.versionable.content.contentId,
									headers: headers,
									load: function (content) {										
										self._displayFileContent(content, innerTbody);				
									},
					                error: function (content) {
					                    console.error('Error: ', content);
					                }
									});
								
								innerTable.appendChild(innerTbody);
								td.appendChild(innerTable);
							}
							else {
								var text = document.createTextNode(this.messages.StreamWorkspaceView_0);
								td.appendChild(text);
							}
							tr.appendChild(td);
							tbody.appendChild(tr);
							table.appendChild(tbody);
							tableDiv.appendChild(table);
							this.fileView.appendChild(tableDiv);
							var br = document.createElement("br");
							this.fileView.appendChild(br);
							var br = document.createElement("br");
							this.fileView.appendChild(br);
						}
					}			
				}
			}
		},
		
		_hasTextContentType: function(versionable) {
			if (versionable.content && versionable.content.contentType) {
				var contentType = versionable.content.contentType;
				if (contentType.indexOf("text/") == 0)
					return true;
				if (contentType == "application/xml")
					return true;
			}
			return false;
		}, 
		
		/**
		 * Display the welcome page of the source control web UI.
		 */
		_setMainPage: function() {
			this.currentPath.innerHTML=this.messages.StreamWorkspaceView_1;

			dojox.data.dom.removeChildren(this.tableHeading);
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML= this.messages.StreamWorkspaceView_2;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			var tr= document.createElement("tr");
			var td= document.createElement("td");
				
			td.setAttribute("colspan", "6");
			td.setAttribute("align", "left");
			td.setAttribute("style", "padding-top: 5px;");
			td.innerHTML= this.messages.StreamWorkspaceView_3;
			tr.appendChild(td);
			this.componentList.appendChild(tr);
		},
		
		/**
		 * Display an error message in the case of a request failure.
		 * 
		 * @param {Object} message
		 */
		displayErrorMessage: function(message) {
			dojox.data.dom.removeChildren(this.componentList);
			dojox.data.dom.removeChildren(this.currentPath);
			dojox.data.dom.removeChildren(this.tableHeading);
			dojox.data.dom.removeChildren(this.changeOverview);
			dojox.data.dom.removeChildren(this.fileView);
			dojox.data.dom.removeChildren(this.flowTargets);
			var div = document.createElement("div");
			div.className = "errorMessage";
			var text = document.createTextNode(message);
			div.appendChild(text);
			this.currentPath.appendChild(div);
		},
		
		/**
		 * Set the current component history paging size.
		 * 
		 * @param {Object} n
		 */
		setHistoryPageSize: function(n) {
			this.currentConfigPageSize = n;
		},
		
		/**
		 * Scroll to the top of the page - Called after each view update.
		 */
		scrollToTop: function() {
			window.scroll(0,0);
		},
		
		/**
		 * Scroll to the specified diff in the change set.
		 * 
		 * @param {Object} diffId
		 */
		scrollToDiffInChangeset: function(diffId) {
			dijit.scrollIntoView(dojo.byId(diffId));
		},
		
		/**
		 * Display the selected change set (information + diffs).
		 * 
		 * @param {Object} changeSetDTO
		 * @param {Object} fromHistory
		 */
		_displayChangeSet: function(changeSetDTO, fromHistory) {
			var workspace = changeSetDTO.workspace;
			var component = changeSetDTO.component;
			
			var author = changeSetDTO.author;
			var dateModified = changeSetDTO.dateModified;
			var comment = changeSetDTO.comment;
			var reasons = changeSetDTO.reasons;
			var changes = changeSetDTO.changes;
			
			this._buildChangeSetNavigationPath(workspace, component);
			
			//////////////////////////////////////////////
			// Change set information summary
			//////////////////////////////////////////////
			
			var table = document.createElement("table");
			table.className = "change-overview";
			var tbody = document.createElement("tbody");
			
			//Modification date entry
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_4;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			var modifiedDateStr = ScmUtil._getDateString(dateModified);
			var text = document.createTextNode(modifiedDateStr);
			td.appendChild(text);
			tr.appendChild(td);
			tbody.appendChild(tr);
			
			//Author entry
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_5;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			td.appendChild(ScmUtil.createContributorElement(author));
			tr.appendChild(td);
			tbody.appendChild(tr);
			
			//Comment entry
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_6;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			if(comment && comment.length > 0) {
				var text = document.createTextNode(comment);
				td.appendChild(text);
			}
			else {
				td.innerHTML = this.messages.StreamWorkspaceView_7;
			}
			tr.appendChild(td);
			tbody.appendChild(tr);
			
			//Reasons entry (work-items)
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_8;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			if(reasons && reasons.length>0) {
				for(var i=0; i<reasons.length; i++) {
					var ul = document.createElement("ul");
					ul.className = "changes";
					var li = document.createElement("li");
					li.appendChild(ScmUtil.createReasonElement(reasons[i]));
					ul.appendChild(li);
					td.appendChild(ul);
					tr.appendChild(td);
					tbody.appendChild(tr);
					var tr = document.createElement("tr");
					var td = document.createElement("td");
					tr.appendChild(td);
					var td = document.createElement("td");
					td.className = "value";
				}
			}
			else {
				td.innerHTML = this.messages.StreamWorkspaceView_9;
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
			
			//Changes entry (Files and folders impacted in the change set)
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_10;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			if(changes && changes.length>0) {	
				var numDiffs = 0;
				for(var i=0; i<changes.length; i++) {
					var ul = document.createElement("ul");
					ul.className = "changes";
					var isNone = (changes[i].changeType & this.NONE) != 0;
					var isAdded = (changes[i].changeType & this.ADD) != 0;
					var isModified = (changes[i].changeType & this.MODIFY) != 0;
					var isRenamed = (changes[i].changeType & this.RENAME) != 0;
					var isReparented = (changes[i].changeType & this.REPARENT) != 0;
					var isDeleted = (changes[i].changeType & this.DELETE) != 0;
					
					var li = document.createElement("li");
					var div = document.createElement("div");
					
					li.appendChild(div);
					var img = document.createElement("img");
					if(changes[i].itemType == "com.ibm.team.scm.Folder") {
						img.src = ScmUtil.getIconSrc("folder");
					}
					else {
						img.src = ScmUtil.getIconSrc("file");
					}
					img.className = "headerIcon";
					var anchor= document.createElement("a");
					anchor.href= (ScmUtil.getViewChangeSetItemURI(workspace.itemId, component.itemId, changes[i].itemType, changes[i].itemId, changes[i].afterStateId));		
					anchor.title= this.messages.StreamWorkspaceView_11;
					var text= document.createTextNode(changes[i].path);
					anchor.appendChild(img);
					anchor.appendChild(text);
					li.appendChild(anchor);
					if(isNone) {
						
					}
					//Add the change context tags
					else {
						if(isAdded) {
							var span = document.createElement("span");
							var text= document.createTextNode(this.messages.StreamWorkspaceView_57);
							span.className = "added";
							span.appendChild(text);
							li.appendChild(span);
						}
						if(isModified) {	//Contains a link to the diff of the modified file
							var anchor = document.createElement("a");
							anchor.className = "modified";
							var diffId = "diff" + numDiffs;
							anchor.href = ScmUtil.getChangeSetURIWithLocation(workspace.itemId, changeSetDTO.itemId, fromHistory, diffId);
							anchor.title = this.messages.StreamWorkspaceView_12;
							var text = document.createTextNode(this.messages.StreamWorkspaceView_13);
							anchor.appendChild(text);
							li.appendChild(anchor);
							numDiffs++;
						}
						if(isRenamed) {
							var span = document.createElement("span");
							var text= document.createTextNode(this.messages.StreamWorkspaceView_14);
							span.className = "renamed";
							span.appendChild(text);
							li.appendChild(span);
						}
						if(isReparented) {
							var span = document.createElement("span");
							var text= document.createTextNode(this.messages.StreamWorkspaceView_15);
							span.className = "reparented";
							span.appendChild(text);
							li.appendChild(span);
						}
						if(isDeleted) {
							var span = document.createElement("span");
							var text= document.createTextNode(this.messages.StreamWorkspaceView_16);
							span.className = "deleted";
							span.appendChild(text);
							li.appendChild(span);
						}
					}
					ul.appendChild(li);
					td.appendChild(ul);
					tr.appendChild(td);
					tbody.appendChild(tr);
					var tr = document.createElement("tr");
					var td = document.createElement("td");
					tr.appendChild(td);
					var td = document.createElement("td");
					td.className = "value";
				}
			}
			else {
				td.innerHTML = this.messages.StreamWorkspaceView_9;
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
			
			table.appendChild(tbody);
			this.changeOverview.appendChild(table);
			
			//////////////////////////////////////////////
			// Change set file diffs
			//////////////////////////////////////////////
			
			if (changeSetDTO.changes) {
				var diffNum = 0;
				for (var i = 0; i < changeSetDTO.changes.length; i++) {
					if (changeSetDTO.changes[i].diff) {
						/////////////////////////////
						// Diff header
						/////////////////////////////
						var diff = changeSetDTO.changes[i].diff;
						var changes = ScmUtil.parseDiff(diff);
						
						var tableDiv = document.createElement("div");
						var table = document.createElement("table");
						table.className = "diffTable";
						var thead = document.createElement("thead");
						var tr = document.createElement("tr");
						var th = document.createElement("th");
						th.setAttribute("width", "100%");
						th.innerHTML = this.messages.StreamWorkspaceView_17;
						
						var text = document.createTextNode(changeSetDTO.changes[i].path);
						th.appendChild(text);
						
						var text = document.createTextNode(this.messages.StreamWorkspaceView_18);
						th.appendChild(text);
						
						var anchor= document.createElement("a");
						anchor.href= (ScmUtil.getViewChangeSetItemURI(workspace.itemId, component.itemId, changeSetDTO.changes[i].itemType, changeSetDTO.changes[i].itemId, changeSetDTO.changes[i].beforeStateId));	
						anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_19, [ changeSetDTO.changes[i].path ]);
						var text= document.createTextNode(this.messages.StreamWorkspaceView_20);
						anchor.appendChild(text);
						th.appendChild(anchor);
						
						var text= document.createTextNode(this.messages.StreamWorkspaceView_21);
						th.appendChild(text);
						
						var anchor= document.createElement("a");
						anchor.href= (ScmUtil.getViewChangeSetItemURI(workspace.itemId, component.itemId, changeSetDTO.changes[i].itemType, changeSetDTO.changes[i].itemId, changeSetDTO.changes[i].afterStateId));	
						anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_22, [ changeSetDTO.changes[i].path ]);
						var text= document.createTextNode(this.messages.StreamWorkspaceView_23);
						anchor.appendChild(text);
						th.appendChild(anchor);
						
						tr.appendChild(th);
						thead.appendChild(tr);
						table.appendChild(thead);
						
						/////////////////////////////
						// Diff body
						/////////////////////////////
						var tbody = this._buildFileDiffTable(changes);
						tableDiv.id = "diff" + diffNum;
						diffNum++;
						table.appendChild(tbody);
						tableDiv.appendChild(table);
						this.fileView.appendChild(tableDiv);
						var br = document.createElement("br");
						this.fileView.appendChild(br);
						var br = document.createElement("br");
						this.fileView.appendChild(br);
					}
				}
			}		
		},
		
		/**
		 * Display the selected change set item (file/folder at specific state).
		 * 
		 * @param {Object} changeSetItem
		 */
		_displayChangeSetItem: function (changeSetItem) {
			if(changeSetItem.versionable["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder") {
				this._displayChangeSetFolderItem(changeSetItem);
			}
			else {
				this._displayChangeSetFileItem(changeSetItem);
			}
		},
		
		/**
		 * Display the selected change set folder item.
		 * 
		 * @param {Object} changeSetItem
		 */
		_displayChangeSetFolderItem: function(changeSetItem) {
			if(changeSetItem.children) {
				this._buildSubElementNavigationPath(changeSetItem.workspace, changeSetItem.component, changeSetItem.path);
				this._addVersionLabel(changeSetItem.versionable.stateId);
				this._addHistoryLink(changeSetItem);
				this._displaySubElements(changeSetItem);
			}
			else {
				this._buildSubElementNavigationPath(changeSetItem.workspace, changeSetItem.component, changeSetItem.path);
				this._addVersionLabel(changeSetItem.versionable.stateId);
				this._addHistoryLink(changeSetItem);
				this._displayEmptyDirectory(changeSetItem.path);
			}	
		},
		
		/**
		 * Display the selected change set file item (a specific state).
		 * 
		 * @param {Object} changeSetItem
		 */
		_displayChangeSetFileItem: function(changeSetItem) {
			
			this._buildSubElementNavigationPath(changeSetItem.workspace, changeSetItem.component, changeSetItem.path, "true");
			
			this._addVersionLabel(changeSetItem.versionable.stateId);
			
			this._addHistoryAndDownloadLinks(changeSetItem);
			
			var tableDiv = document.createElement("div");
			var table = document.createElement("table");
			table.className = "fileContent";
			var thead = document.createElement("thead");
			var tr = document.createElement("tr");
			tr.className = "item-name";
			var th = document.createElement("th");
			th.className = "item-name";
			var text = document.createTextNode(changeSetItem.versionable.name);
			th.appendChild(text);
			tr.appendChild(th);
			thead.appendChild(tr);
			
			table.appendChild(thead);

			var tbody = document.createElement("tbody");
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			
			//Inner table is used since COLSPAN is not supported properly in IE.
			var innerTable = document.createElement("table");
			innerTable.className = "file";
			var thead = document.createElement("thead");
			var innerTr = document.createElement("tr");
			innerTr.className = "line-heading";
			var th = document.createElement("th");
			th.className = "line-heading";
			th.innerHTML = this.messages.StreamWorkspaceView_24;
			innerTr.appendChild(th);
			var th = document.createElement("th");
			th.className = "content-heading";
			innerTr.appendChild(th);
			thead.appendChild(innerTr);
			innerTable.appendChild(thead);
			
			var innerTbody = document.createElement("tbody");
			
			var self = this;
			var xhr = dojo.xhrGet({
				url:"../../service/com.ibm.team.repository.common.transport.ITeamContentService/" + changeSetItem.versionable.content.contentId,
				
				load: function (content) {								
					self._displayFileContent(content, innerTbody);					
				},
                error: function (content) {
                    console.error('Error: ', content);
                }
				});
			
			innerTable.appendChild(innerTbody);
			td.appendChild(innerTable);
			tr.appendChild(td);
			tbody.appendChild(tr);
			table.appendChild(tbody);
			tableDiv.appendChild(table);
			this.fileView.appendChild(tableDiv);
			var br = document.createElement("br");
			this.fileView.appendChild(br);
			var br = document.createElement("br");
			this.fileView.appendChild(br);
		},
		
		/**
		 * TODO implement once snapshotDTO is returned by REST call 
		 * @param {Object} DTO
		 */
		_displaySnapshots: function(DTO) {
			
		},
		
		/**
		 * Add a version label to the breadcrumbs (for files accessed at specific states i.e from change sets).
		 * 
		 * @param {Object} version
		 */
		_addVersionLabel: function(version) {
			var text = document.createTextNode(" > ");
			this.currentPath.appendChild(text);
			var text = document.createTextNode(version);
			this.currentPath.appendChild(text);
		},
		
		/**
		 * Build the body of the table containing the change set diff.
		 * 
		 * @param {Object} changes
		 */
		_buildFileDiffTable: function(changes) {
			var tbody = document.createElement("tbody");

			for (var i = 0; i < changes.length; i++) {
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.setAttribute("width", "100%");
				
				var div1 = document.createElement("div");
				div1.className = "diff-content";
				
				var count = 0;
				var lines = [];
				var cr = 0;
				var lf = 0;
				var crlf = 0;

				var j = 0;
				while (count < changes[i].length) {	
					//Check for the newline separator being used in the diff.
					if (cr != -1) {
						cr = changes[i].indexOf("\r", count);
					}
					if (lf != -1) {
						lf = changes[i].indexOf("\n", count);
					}
					if (crlf != -1) {
						crlf = changes[i].indexOf("\r\n", count);
					}
					
					//Extract the line in the diff.
					if (cr != -1 || lf != -1 || crlf != -1) {
						if (crlf != -1 && crlf <= cr && crlf < lf) {
							lines[j] = changes[i].slice(count, crlf);
							count = crlf + 2;
							j++;
						}
						else if(cr!=-1 && ((lf != -1 && cr < lf) || lf == -1)) {
							lines[j] = changes[i].slice(count, cr);
							count = cr + 1;
							j++;
						}
						else if(lf!=-1) {
							lines[j] = changes[i].slice(count, lf);
							count = lf + 1;
							j++;
						}
					}
					else {	//Last line in diff reached.
						lines[j] = changes[i].slice(count, changes[i].length);
						count = changes[i].length;
						j++;
					}
					
					var div = document.createElement("div");
					
					//Highlight the diff sections
					if (lines[j-1].charAt(0) == '@') {
						div.className = "diff3";
					}
					else if (lines[j-1].charAt(0) == '-') {
						div.className = "diff1";
					}
					else if (lines[j-1].charAt(0) == '+') {
						div.className = "diff2";
					}
					else if (lines[j-1]=="\\ No newline at end of file") {
						div.className = "diff4";
					}
					else {
						div.className = "diff";
					}
					var text = document.createTextNode(lines[j-1]);
					div.appendChild(text);
					div1.appendChild(div);
				}
				
				td.appendChild(div1);
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
			return tbody;
		},
		
		/**
		 * Construct the breadcrumbs for a change set.
		 * 
		 * @param {Object} workspace
		 * @param {Object} component
		 */
		_buildChangeSetNavigationPath: function(workspace, component) {
			var img = document.createElement("img");
			var anchor= document.createElement("a");
			if(workspace.stream == "true" || workspace.stream) {
				img.src = ScmUtil.getIconSrc("stream");
				anchor.href= (ScmUtil.getBrowseStreamURI(workspace.itemId));
			}
			else if(workspace.stream == "false" || !workspace.stream) {
				img.src = ScmUtil.getIconSrc("workspace");
				anchor.href= (ScmUtil.getBrowseWorkspaceURI(workspace.itemId));		
			}
			img.className = "headerIcon";
//			anchor.title= workspace.name;
			var text= document.createTextNode(workspace.name);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("component");
			img.className = "headerIcon";
			var anchor= document.createElement("a");
			anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, "/"));		
//			anchor.title= component.name;
			var text= document.createTextNode(component.name);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("history");
			img.className = "headerIcon";
			var anchor= document.createElement("a");
			anchor.href= (ScmUtil.getComponentHistoryURI(workspace.itemId, component.itemId, this.PAGE_SIZE));		
			anchor.title= this.messages.StreamWorkspaceView_26;
			var text= document.createTextNode(this.messages.StreamWorkspaceView_27);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);		
			
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			var text= document.createTextNode(this.messages.StreamWorkspaceView_28);
			this.currentPath.appendChild(text);
				
		},
		
		/**
		 * Display the history of a component or item.
		 * 
		 * @param {Object} historyDTO
		 * @param {Object} historyType
		 */
		_displayHistory: function(historyDTO, historyType) {
			var workspace = historyDTO.workspace;
			var component = historyDTO.component;
			
			/////////////////////////////////
			// Breadcrumbs
			/////////////////////////////////
			var img = document.createElement("img");
			var anchor= document.createElement("a");
			if(workspace.stream == "true" || workspace.stream) {
				img.src = ScmUtil.getIconSrc("stream");
				anchor.href= (ScmUtil.getBrowseStreamURI(workspace.itemId));
			}
			else if(workspace.stream == "false" || !workspace.stream) {
				img.src = ScmUtil.getIconSrc("workspace");
				anchor.href= (ScmUtil.getBrowseWorkspaceURI(workspace.itemId));		
			}
			img.className = "headerIcon";
//			anchor.title= workspace.name;
			var text= document.createTextNode(workspace.name);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("component");
			img.className = "headerIcon";
			var anchor= document.createElement("a");
			anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, "/"));		
//			anchor.title= component.name;
			var text= document.createTextNode(component.name);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			
			if(historyType == "item") {
				var isHistoryBreadCrumb = true;
				var isFile = true;
				if (historyDTO.itemType == "com.ibm.team.scm.Folder") {
					isFile = false;
				}
				//Build the file/folder path part of the breadcrumb
				this._parseAndDisplayPath(historyDTO.workspace, historyDTO.component, historyDTO.path, isFile, isHistoryBreadCrumb);
				var text = document.createTextNode(" > ");
				this.currentPath.appendChild(text);
			}
			
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("history");
			img.className = "headerIcon";
			var text = document.createTextNode(this.messages.StreamWorkspaceView_27);
			this.currentPath.appendChild(img);
			this.currentPath.appendChild(text);
			
			////////////////////////////////////////
			//History content - list of change sets
			////////////////////////////////////////
			
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.setAttribute("width", "100%");	
			if (historyType == "item") {
				th.innerHTML = this.messages.StreamWorkspaceView_29;
			}
			else {
				th.innerHTML = this.messages.StreamWorkspaceView_30;
			}
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			var changeSets = historyDTO.changeSets;
			
			for(var i=0; i<changeSets.length; i++) {
				//Global row and data cell containing each change set entry
				var tr= document.createElement("tr");
				var td= document.createElement("td");
				td.setAttribute("width", "100%");
				var div= document.createElement("div");
				if(i%2==0) {
					div.className = "history-div-even";
				}
				else {
					div.className = "history-div-odd";
				}
				
				//Inner table containing the "Comment" field of a change set
				var table = document.createElement("table");
				table.setAttribute("width", "100%");
				table.className = "nested-table";
				var tbody = document.createElement("tbody");
				var tri = document.createElement("tr");
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "5%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_31;
				tri.appendChild(tdi);
								
				var tdi = document.createElement("td");
				var anchor = document.createElement("a");
				anchor.href = (ScmUtil.getChangeSetURI(workspace.itemId, changeSets[i].itemId, "1"));
				anchor.title = this.messages.StreamWorkspaceView_32;
				if (changeSets[i].comment && changeSets[i].comment != "" && changeSets[i].comment.length > 0) {
					var text = document.createTextNode(changeSets[i].comment);
				}
				else {			
					var text = document.createTextNode(this.messages.StreamWorkspaceView_33);
				}
				anchor.appendChild(text);
				tdi.appendChild(anchor);
				
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing the "Reasons" (work-items) field of a change set
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");	
				
				if(changeSets[i].reasons) {
					for(var j=0; j<changeSets[i].reasons.length; j++) {
						var tri = document.createElement("tr");
						var tdi = document.createElement("td");
						tdi.setAttribute("width", "7%");
						if(j==0) {
							tdi.className = "history-inner-table";
							tdi.innerHTML = this.messages.StreamWorkspaceView_34;
						}
						else {
							tdi.innerHTML = "";
						}
						tri.appendChild(tdi);
						var tdii = document.createElement("td");
						tdii.appendChild(ScmUtil.createReasonElement(changeSets[i].reasons[j]));
						tri.appendChild(tdii);
						tbody.appendChild(tri);
						table.appendChild(tbody);
					}
				}
				else {
					var tri = document.createElement("tr");
					var tdi = document.createElement("td");
					tdi.setAttribute("width", "7%");
					tdi.className = "history-inner-table";
					tdi.innerHTML = this.messages.StreamWorkspaceView_34;
					tri.appendChild(tdi);
					
					var tdi = document.createElement("td");
					tdi.innerHTML = this.messages.StreamWorkspaceView_35;
					tri.appendChild(tdi);
					
					tbody.appendChild(tri);
					table.appendChild(tbody);
				}
				
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing "Created by", "Added by", "Added on", "Modified on" fields of a change set
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");
				var tri = document.createElement("tr");
				
				//"Created by" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_36;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				tdi.setAttribute("width", "15%");
				tdi.appendChild(ScmUtil.createContributorElement(changeSets[i].author));
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				//"Modified on" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_37;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				var modifiedDateStr= ScmUtil._getDateString(changeSets[i].dateModified);
				var text = document.createTextNode(modifiedDateStr);
				tdi.appendChild(text);
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				//"Added by" field
				var tri = document.createElement("tr");
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_38;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				tdi.setAttribute("width", "15%");
				tdi.appendChild(ScmUtil.createContributorElement(changeSets[i].addedBy));
				tri.appendChild(tdi);
				
				//"Added on" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_39;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				var addedDateStr= ScmUtil._getDateString(changeSets[i].dateAdded);
				var text = document.createTextNode(addedDateStr);
				tdi.appendChild(text);
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				tr.appendChild(td);
				this.componentList.appendChild(tr);
			}
			
			if (historyType == "component") {
				/* 	//This section is commented out until functionality from the server is provided to detect
				 	//whether or not there are more older and newer change sets to display (to activate or
					 //deactivate the "older" and "newer" links.
				 if(historyDTO.previous && historyDTO.next) {
				 	this._addPageNavigation(workspace, component, this.PAGE_SIZE, changeSets[0], changeSets[changeSets.length - 1]);
				 }
				 else if(historyDTO.next) {
				 	this._addPageNavigation(workspace, component, this.PAGE_SIZE, changeSets[0], null);
				 }
				 else if(historyDTO.previous) {
				 	this._addPageNavigation(workspace, component, this.PAGE_SIZE, null, changeSets[changeSets.length - 1]);
				 }
				 */
				this._addPageNavigation(workspace, component, this.currentConfigPageSize, changeSets[0], changeSets[changeSets.length - 1]);
			}
		},
		
		/**
		 * Display the page navigation links ("Older" and "Newer").
		 * 
		 * @param {Object} workspace
		 * @param {Object} component
		 * @param {Object} n
		 * @param {Object} first
		 * @param {Object} last
		 */
		_addPageNavigation: function(workspace, component, n, first, last) {
			
			var div = document.createElement("div");
			div.className = "paging-navigation";
			
			var newerLabel = "\u00AB " + dojo.string.substitute(this.messages.StreamWorkspaceView_58);
			var olderLabel = dojo.string.substitute(this.messages.StreamWorkspaceView_59) + " \u00BB";

			if (first) {
				var anchor = document.createElement("a");
				anchor.rel = "prev";
				anchor.href = ScmUtil.getComponentHistoryURIWithFirst(workspace.itemId, component.itemId, first.itemId, n);
				var text = document.createTextNode(newerLabel);
				anchor.appendChild(text);
				div.appendChild(anchor);
			}
			else {
				var span = document.createElement("span");
				span.className = "inactiveLink";
				var text = document.createTextNode(newerLabel);
				span.appendChild(text);
				div.appendChild(span);
			}

			var text = document.createTextNode(this.messages.StreamWorkspaceView_21);
			div.appendChild(text);
			
			if (last) {
				var anchor = document.createElement("a");
				anchor.rel = "next";
				anchor.href = ScmUtil.getComponentHistoryURIWithLast(workspace.itemId, component.itemId, last.itemId, n);
				var text = document.createTextNode(olderLabel);
				anchor.appendChild(text);
				div.appendChild(anchor);
			}
			else {
				var span = document.createElement("span");
				span.className = "inactiveLink";
				var text = document.createTextNode(olderLabel);
				span.appendChild(text);
				div.appendChild(span);
			}
			
			this.fileView.appendChild(div);
		},
		
		/**
		 * Display team activity (list of active change sets).
		 * 
		 * @param {Object} changeSets
		 */
		_displayTeamChanges: function(changeSets) {
			var text= document.createTextNode(this.messages.StreamWorkspaceView_40);
			this.currentPath.appendChild(text);
			
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.setAttribute("width", "100%");
			th.innerHTML = this.messages.StreamWorkspaceView_41;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			if (changeSets == "none") {
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.setAttribute("colspan", "6");
				td.setAttribute("align", "left");
				td.setAttribute("style", "padding-top: 5px;");
				td.innerHTML= this.messages.StreamWorkspaceView_42;
				tr.appendChild(td);
				this.componentList.appendChild(tr);
			}
			else {
				this._buildChangeSetTable(changeSets, false);
			}
		},
		
		/**
		 * Build the change set table for team activity (Generalized to be adapted later for history as well).
		 * 
		 * @param {Object} changeSets
		 * @param {Object} isHistory
		 */
		_buildChangeSetTable: function(changeSets, isHistory) {	
			for (var i = 0; i < changeSets.length; i++) {
				//Global row and data cell for each change set entry
				var tr = document.createElement("tr");
				var td = document.createElement("td");
				td.setAttribute("width", "100%");
				var div = document.createElement("div");
				if (i % 2 == 0) {
					div.className = "history-div-even";
				}
				else {
					div.className = "history-div-odd";
				}
				
				//Inner table containing the "Comment" field
				var table = document.createElement("table");
				table.setAttribute("width", "100%");
				table.className = "nested-table";
				var tbody = document.createElement("tbody");
				var tri = document.createElement("tr");
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "5%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_31;
				tri.appendChild(tdi);
				
				var tdi = document.createElement("td");
				var anchor = document.createElement("a");
				if (isHistory) {
					anchor.href = (ScmUtil.getChangeSetURI(changeSets[i].workspace.itemId, changeSets[i].itemId, "1"));
				}
				else {
					anchor.href = (ScmUtil.getChangeSetURI(changeSets[i].workspace.itemId, changeSets[i].itemId, "0"));
				}
				anchor.title = this.messages.StreamWorkspaceView_43;
				if (changeSets[i].comment && changeSets[i].comment != "" && changeSets[i].comment.length > 0) {
					var text = document.createTextNode(changeSets[i].comment);	
				}
				else {
					var text = document.createTextNode(this.messages.StreamWorkspaceView_33);
				}
				anchor.appendChild(text);
				tdi.appendChild(anchor);
				
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing the "Reasons" field
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");
				
				if (changeSets[i].reasons) {
					for (var j = 0; j < changeSets[i].reasons.length; j++) {
						var tri = document.createElement("tr");
						var tdi = document.createElement("td");
						tdi.setAttribute("width", "7%");
						if (j == 0) {
							tdi.className = "history-inner-table";
							tdi.innerHTML = this.messages.StreamWorkspaceView_34;
						}
						else {
							tdi.innerHTML = "";
						}
						tri.appendChild(tdi);
						var tdii = document.createElement("td");
						tdii.appendChild(ScmUtil.createReasonElement(changeSets[i].reasons[j]));
						tri.appendChild(tdii);
						tbody.appendChild(tri);
						table.appendChild(tbody);
					}
				}
				else {
					var tri = document.createElement("tr");
					var tdi = document.createElement("td");
					tdi.setAttribute("width", "7%");
					tdi.className = "history-inner-table";
					tdi.innerHTML = this.messages.StreamWorkspaceView_34;
					tri.appendChild(tdi);
					
					var tdi = document.createElement("td");
					tdi.innerHTML = this.messages.StreamWorkspaceView_35;
					tri.appendChild(tdi);
					
					tbody.appendChild(tri);
					table.appendChild(tbody);
				}
				
				div.appendChild(table);
				td.appendChild(div);
				
				//Inner table containing "Created by" and "Modified on" fields
				var table = document.createElement("table");
				var tbody = document.createElement("tbody");
				table.className = "nested-table";
				table.setAttribute("width", "50%");
				var tri = document.createElement("tr");
				
				//"Created by" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_36;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				tdi.setAttribute("width", "15%");
				tdi.appendChild(ScmUtil.createContributorElement(changeSets[i].author));
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				//"Modified on" field
				var tdi = document.createElement("td");
				tdi.className = "history-inner-table";
				tdi.setAttribute("width", "7%");
				tdi.innerHTML = this.messages.StreamWorkspaceView_37;
				tri.appendChild(tdi);
				var tdi = document.createElement("td");
				var modifiedDateStr = ScmUtil._getDateString(changeSets[i].dateModified);
				var text = document.createTextNode(modifiedDateStr);
				tdi.appendChild(text);
				tri.appendChild(tdi);
				tbody.appendChild(tri);
				
				if (isHistory) {
					//"Added by" field
					var tri = document.createElement("tr");
					var tdi = document.createElement("td");
					tdi.className = "history-inner-table";
					tdi.setAttribute("width", "7%");
					tdi.innerHTML = this.messages.StreamWorkspaceView_38;
					tri.appendChild(tdi);
					var tdi = document.createElement("td");
					tdi.setAttribute("width", "15%");
					var text = document.createTextNode(changeSets[i].addedBy);
					tdi.appendChild(text);
					tri.appendChild(tdi);
					
					//"Added on" field
					var tdi = document.createElement("td");
					tdi.className = "history-inner-table";
					tdi.setAttribute("width", "7%");
					tdi.innerHTML = this.messages.StreamWorkspaceView_39;
					tri.appendChild(tdi);
					var tdi = document.createElement("td");
					var addedDateStr = ScmUtil._getDateString(changeSets[i].dateAdded);
					var text = document.createTextNode(addedDateStr);
					tdi.appendChild(text);
					tri.appendChild(tdi);
					tbody.appendChild(tri);
				}
				
				table.appendChild(tbody);
				div.appendChild(table);
				td.appendChild(div);
				
				tr.appendChild(td);
				this.componentList.appendChild(tr);
			}
		},
		
		/**
		 * Display an empty directory.
		 * 
		 * @param {Object} path
		 */
		_displayEmptyDirectory: function(path) {
			dojox.data.dom.removeChildren(this.tableHeading);
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML= this.messages.StreamWorkspaceView_44;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			var tr= document.createElement("tr");
			var td= document.createElement("td");
				
			td.setAttribute("colspan", "6");
			td.setAttribute("align", "left");
			td.setAttribute("style", "padding-top: 5px;");
			td.innerHTML= this.messages.StreamWorkspaceView_45;
			tr.appendChild(td);
			this.componentList.appendChild(tr);
		},
		
		/**
		 * Add the workspace info including: description and links to the flow targets.
		 * Also adds a link to view the snapshots of a workspace.
		 * 
		 * @param {Object} workspaceDTO
		 */
		_addWorkspaceInfo: function(workspaceDTO) {
			//////////////////////////////////////////////
			// Workspace information summary
			//////////////////////////////////////////////
			
			//Show snapshots entry
			/**
			 * TODO: Uncomment this section once snapshotDTO is implemented
			 */
			/*
			var table = document.createElement("table");
			table.className = "show-options";
			var tbody = document.createElement("tbody");
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.innerHTML = "Show:";
			td.className = "history-header";
			tr.appendChild(td);
			var td = document.createElement("td");
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("snapshot");
			img.className = "headerIcon";
			//Add a link to view the snapshots of workspace/stream
			var anchor= document.createElement("a");
			anchor.href= ScmUtil.getViewSnapshotsURI(workspaceDTO.workspace.itemId);
			anchor.title= "Baseline Sets";
			var text= document.createTextNode("Snapshots");
			anchor.appendChild(img);
			anchor.appendChild(text);
			td.appendChild(anchor);
			tr.appendChild(td);
			tbody.appendChild(tr);
			
			table.appendChild(tbody);
			this.changeOverview.appendChild(table);	*/
				
			
			//Workspace information section
			var table = document.createElement("table");
			table.className = "change-overview";
			var tbody = document.createElement("tbody");
			
			//Description entry
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = this.messages.StreamWorkspaceView_46;
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			if (workspaceDTO.workspace.description != "") {
				var text = workspaceDTO.workspace.description;
				var lines = text.split(/\n/);
				for (var i=0; i < lines.length; i++) {
					if (i > 0) {
						td.appendChild(document.createElement("br"));
					}
					td.appendChild(document.createTextNode(lines[i]));
				}
			}
			else {
				var text = document.createTextNode(this.messages.StreamWorkspaceView_47);
				td.appendChild(text);
			}
			tr.appendChild(td);
			tbody.appendChild(tr);
/*			
			//Owner entry
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			td.className = "header";
			td.innerHTML = "Owner: ";
			tr.appendChild(td);
			var td = document.createElement("td");
			td.className = "value";
			if (workspaceDTO.workspace.owner != null) {
				var text = document.createTextNode(workspaceDTO.workspace.owner.itemId);
				td.appendChild(text);
			}
			else {
				var text = document.createTextNode("<None>");
				td.appendChild(text);
			}
			tr.appendChild(td);
			tbody.appendChild(tr);
*/			
			table.appendChild(tbody);
			this.changeOverview.appendChild(table);
			
			this._addFlowTargetsSection(workspaceDTO);
		},
		
		_addFlowTargetsSection: function(workspaceDTO) {
			var table = document.createElement("table");
			table.className = "stream";
			var thead = document.createElement("thead");
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML= this.messages.StreamWorkspaceView_48;
			tr.appendChild(th);
			thead.appendChild(tr);
			table.appendChild(thead);
			
			var tbody = document.createElement("tbody");
			var self = this;
			if (workspaceDTO.flowTargets) {
				dojo.forEach(workspaceDTO.flowTargets, function(x) {
					var tr= document.createElement("tr");
					var td= document.createElement("td");
					td.setAttribute("colspan", "6");
					td.setAttribute("align", "left");
					td.setAttribute("style", "padding-top: 5px;");
					var img = document.createElement("img");
					img.src = ScmUtil.getIconSrc("stream");
					img.className = "icon";
					
					var anchor= document.createElement("a");
					anchor.href= (ScmUtil.getBrowseStreamURI(x.itemId));
//					anchor.title= x.name;
					var text= document.createTextNode(x.name);
					anchor.appendChild(text);
					
					td.appendChild(img);
					td.appendChild(anchor);
					
					tr.appendChild(td);
					tbody.appendChild(tr);
				});
			}
			else {
				var tr= document.createElement("tr");
				var td= document.createElement("td");
				td.setAttribute("colspan", "6");
				td.setAttribute("align", "left");
				td.setAttribute("style", "padding-top: 5px;");
				td.innerHTML= this.messages.StreamWorkspaceView_49;
				
				tr.appendChild(td);
				tbody.appendChild(tr);
			}
			table.appendChild(tbody);
			var br = document.createElement("br");
			this.flowTargets.appendChild(br);
			this.flowTargets.appendChild(table);
		},
		
		/**
		 * Display the components of a selected repository workspace.
		 * 
		 * @param {Object} components
		 * @param {Object} workspaceId
		 */
		_displayWorkspaceComponents: function(components, workspaceId) {
			dojox.data.dom.removeChildren(this.tableHeading);
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML= this.messages.StreamWorkspaceView_50;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			var self = this;
			if (components) {
				dojo.forEach(components, function(x) {
					var tr= document.createElement("tr");
					var td= document.createElement("td");
					td.setAttribute("colspan", "6");
					td.setAttribute("align", "left");
					td.setAttribute("style", "padding-top: 5px;");
					var img = document.createElement("img");
					img.src = ScmUtil.getIconSrc("component");
					img.className = "icon";
					
					var anchor= document.createElement("a");
					anchor.href= (ScmUtil.getBrowseElementURI(workspaceId, x.itemId, "/"));
//					anchor.title= x.name;
					var text= document.createTextNode(x.name);
					anchor.appendChild(text);
					
					td.appendChild(img);
					td.appendChild(anchor);
					
					tr.appendChild(td);
					self.componentList.appendChild(tr);
				});
			}
			else {
				var tr= document.createElement("tr");
				var td= document.createElement("td");
				td.setAttribute("colspan", "6");
				td.setAttribute("align", "left");
				td.setAttribute("style", "padding-top: 5px;");
				td.innerHTML= this.messages.StreamWorkspaceView_51;
				
				tr.appendChild(td);
				self.componentList.appendChild(tr);
			}
		},
		
		/**
		 * Add a "History" link under the "Show" field for components, folders, or files.
		 * 
		 * @param {Object} subElements
		 */
		_addHistoryLink: function(subElements) {
			var workspaceId = subElements.workspace.itemId;
			var componentId = subElements.component.itemId;

			var table = document.createElement("table");
			table.className = "show-options";
			var tbody = document.createElement("tbody");
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			
			td.innerHTML = this.messages.StreamWorkspaceView_52;
			td.className = "history-header";
			tr.appendChild(td);
			var td = document.createElement("td");
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("history");
			img.className = "headerIcon";
			if (subElements.path == "/") {		
				//Add a link to the history of the component
				var anchor= document.createElement("a");
				anchor.href= (ScmUtil.getComponentHistoryURI(workspaceId, componentId, this.PAGE_SIZE));
				anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_53, [ subElements.component.name ]);
				var text= document.createTextNode(this.messages.StreamWorkspaceView_54);
				anchor.appendChild(img);
				anchor.appendChild(text);
			}
			else {			
				//Add a link to the history of the item (file or folder)
				var anchor = document.createElement("a");
				anchor.href = (ScmUtil.getItemHistoryURI(workspaceId, componentId, subElements.versionable["com.ibm.team.repository.typeName"] + "/" + subElements.versionable.itemId));
				anchor.title = dojo.string.substitute(this.messages.StreamWorkspaceView_53, [ subElements.versionable.name ]);
				var text = document.createTextNode(this.messages.StreamWorkspaceView_54);
				anchor.appendChild(img);
				anchor.appendChild(text);
			}
			td.appendChild(anchor);
			tr.appendChild(td);
			tbody.appendChild(tr);
			table.appendChild(tbody);
			this.changeOverview.appendChild(table);
		},
		
		/**
		 * Add a "History" and a "Download" link under the "Show" field for files.
		 * 
		 * @param {Object} versionableDTO
		 */
		_addHistoryAndDownloadLinks: function(versionableDTO) {
			var workspaceId = versionableDTO.workspace.itemId;
			var componentId = versionableDTO.component.itemId;

			var table = document.createElement("table");
			table.className = "show-options";
			var tbody = document.createElement("tbody");
			var tr = document.createElement("tr");
			var td = document.createElement("td");
			
			td.innerHTML = this.messages.StreamWorkspaceView_52;
			td.className = "history-header";
			tr.appendChild(td);
			var td = document.createElement("td");
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("history");
			img.className = "headerIcon";
			if (versionableDTO.path == "/") {		
				//Add a link to the history of the component
				var anchor= document.createElement("a");
				anchor.href= (ScmUtil.getComponentHistoryURI(workspaceId, componentId, this.PAGE_SIZE));
				anchor.title= dojo.string.substitute(this.messages.StreamWorkspaceView_53, [ versionableDTO.component.name ]);
				var text= document.createTextNode(this.messages.StreamWorkspaceView_54);
				anchor.appendChild(img);
				anchor.appendChild(text);
			}
			else {			
				//Add a link to the history of the item
				var anchor = document.createElement("a");
				anchor.href = (ScmUtil.getItemHistoryURI(workspaceId, componentId, versionableDTO.versionable["com.ibm.team.repository.typeName"] + "/" + versionableDTO.versionable.itemId));
				anchor.title = dojo.string.substitute(this.messages.StreamWorkspaceView_53, [ versionableDTO.versionable.name ]);
				var text = document.createTextNode(this.messages.StreamWorkspaceView_54);
				anchor.appendChild(img);
				anchor.appendChild(text);
			}
			td.appendChild(anchor);
			tr.appendChild(td);
			
			var td = document.createElement("td");
			var img = document.createElement("img");
			img.src = ScmUtil.getIconSrc("file");
			img.className = "headerIcon";

			var anchor = document.createElement("a");
			anchor.href = ScmUtil.getFileDownloadURI(workspaceId, componentId, versionableDTO.versionable.name, versionableDTO.versionable.itemId, versionableDTO.versionable.stateId);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_55);
			anchor.appendChild(img);
			anchor.appendChild(text);

			td.appendChild(anchor);
			tr.appendChild(td);
			
			tbody.appendChild(tr);
			table.appendChild(tbody);
			this.changeOverview.appendChild(table);
		},
		
		/**
		 * Display the contents of a component or folder.
		 * 
		 * @param {Object} subElements
		 */
		_displaySubElements: function(subElements) {
			var workspaceId = subElements.workspace.itemId;
			var componentId = subElements.component.itemId;
			
			dojox.data.dom.removeChildren(this.tableHeading);
			var tr = document.createElement("tr");
			var th = document.createElement("th");
			th.innerHTML= this.messages.StreamWorkspaceView_44;
			tr.appendChild(th);
			this.tableHeading.appendChild(tr);
			
			var self = this;
			dojo.forEach(subElements.children, function(x) {
				var tr= document.createElement("tr");
				var td= document.createElement("td");
				td.setAttribute("colspan", "6");
				td.setAttribute("align", "left");
				td.setAttribute("style", "padding-top: 5px;");
				var img = document.createElement("img");
				var isFolder = x["com.ibm.team.repository.typeName"] == "com.ibm.team.scm.Folder";
				img.src = ScmUtil.getIconSrc(isFolder ? "folder" : "file");
				img.className = "icon";
				
				var anchor= document.createElement("a");
				//If the selected item is a component.
				if(subElements.path!="/") {
					anchor.href= (ScmUtil.getBrowseElementURI(subElements.workspace.itemId, subElements.component.itemId, subElements.path + "/" + x.name));
				}
				else {
					anchor.href= (ScmUtil.getBrowseElementURI(subElements.workspace.itemId, subElements.component.itemId, subElements.path + x.name));
				}
//				anchor.title= x.name;
				var text= document.createTextNode(x.name);
	
				anchor.appendChild(text);
				td.appendChild(img);
				td.appendChild(anchor);
				
				tr.appendChild(td);
				self.componentList.appendChild(tr);
			});	
		},
		
		/**
		 * Build the breadcrumbs of the path to a workspace.
		 * 
		 * @param {Object} name
		 * @param {Object} id
		 * @param {Object} type
		 */
		_buildWorkspaceNavigationPath: function(name, id, type) {		
			var img = document.createElement("img");
			if(type == "stream") {
				img.src = ScmUtil.getIconSrc("stream");
			}
			else if(type == "workspace") {
				img.src = ScmUtil.getIconSrc("workspace");		
			}
			img.className = "headerIcon";
			this.currentPath.appendChild(img);
			var text= document.createTextNode(name);
			this.currentPath.appendChild(text);
		},
	
		/**
		 * Builds the breadcrumbs of the path to a component or item (file or folder).
		 * 
		 * @param {Object} workspace
		 * @param {Object} component
		 * @param {Object} path
		 * @param {Object} isFile
		 */
		_buildSubElementNavigationPath: function(workspace, component, path, /*optional*/isFile) {
			var img = document.createElement("img");
			var anchor= document.createElement("a");
			if(workspace.stream == "true" || workspace.stream) {
				img.src = ScmUtil.getIconSrc("stream");
				anchor.href= (ScmUtil.getBrowseStreamURI(workspace.itemId));
			}
			else if(workspace.stream == "false" || !workspace.stream) {
				img.src = ScmUtil.getIconSrc("workspace");
				anchor.href= (ScmUtil.getBrowseWorkspaceURI(workspace.itemId));		
			}
			img.className = "headerIcon";
//			anchor.title= workspace.name;
			var text= document.createTextNode(workspace.name);
			anchor.appendChild(img);
			anchor.appendChild(text);
			this.currentPath.appendChild(anchor);
			var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
			this.currentPath.appendChild(text);
			
			//If the path is "/", then the item displayed for browsing is a component.
			if(path=="/") {
				var img = document.createElement("img");
				img.src = ScmUtil.getIconSrc("component");
				img.className = "headerIcon";
				this.currentPath.appendChild(img);
				var text= document.createTextNode(component.name);
				this.currentPath.appendChild(text);
			}
			
			else {
				var img = document.createElement("img");
				img.src = ScmUtil.getIconSrc("component");
				img.className = "headerIcon";
				var anchor= document.createElement("a");
				anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, "/"));		

//				anchor.title= component.name;
				var text= document.createTextNode(component.name);
				anchor.appendChild(img);
				anchor.appendChild(text);
				this.currentPath.appendChild(anchor);
				
				var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
				this.currentPath.appendChild(text);
				
				//Parse the path in order to provide navigation through the various items within it.
				if(isFile) {
					this._parseAndDisplayPath(workspace, component, path, isFile);
				}
				else {		
					this._parseAndDisplayPath(workspace, component, path);
				}
			}
		},
		
		/**
		 * Parse and display the content of a file.
		 * 
		 * @param {Object} encodedContent
		 * @param {Object} tbody
		 */
		_displayFileContent: function(encodedContent, tbody) {
			var count = 0;
			var lines = [];
			var j = 0;
			var cr = 0;
			var lf = 0;
			var crlf = 0;
			while (count < encodedContent.length) {
				var tr = document.createElement("tr");
				var th = document.createElement("th");
				var id = j+1;
				th.id = "line" + id;
				th.innerHTML = id;
				tr.appendChild(th);
				var td = document.createElement("td");
				var div = document.createElement("div");
				div.className = "content";
				td.setAttribute("width", "100%");
				
				if (cr != -1) {
					cr = encodedContent.indexOf("\r", count);
				}
				if (lf != -1) {
					lf = encodedContent.indexOf("\n", count);
				}
				if (crlf != -1) {
					crlf = encodedContent.indexOf("\r\n", count);
				}
				
				if (cr != -1 || lf != -1 || crlf != -1) {
					if (crlf != -1 && crlf <= cr && crlf < lf) {
						lines[j] = encodedContent.slice(count, crlf);
						count = crlf + 2;
					}
					else if(cr!=-1 && ((lf != -1 && cr < lf) || lf == -1)) {
						lines[j] = encodedContent.slice(count, cr);
						count = cr + 1;
					}
					else if(lf!=-1) {
						lines[j] = encodedContent.slice(count, lf);
						count = lf + 1;
					}

					if (lines[j].length > 0) {
						var text = document.createTextNode(lines[j]);
						div.appendChild(text);
						if(count>=encodedContent.length) { //End of file reached (ending with a line separator)
							var divLast = document.createElement("div");
							divLast.innerHTML = " ";
						}
					}
					else {
						div.innerHTML = " ";
					}
					j++;
				}
				else { //End of file reached
					lines[j] = encodedContent.slice(count, encodedContent.length);
					count = encodedContent.length;
					if (lines[j].length > 0) {
						var text = document.createTextNode(lines[j]);
						div.appendChild(text);
					}
					else {
						div.innerHTML = " ";
					}
				}

				td.appendChild(div);
				tr.appendChild(td);
				tbody.appendChild(tr);
				if(divLast) {
					var tr = document.createElement("tr");
					var th = document.createElement("th");
					var id = j+1;
					th.id = "line" + id;
					th.innerHTML = id;
					tr.appendChild(th);
					var td = document.createElement("td");
					divLast.className = "content";
					td.setAttribute("width", "100%");
					td.appendChild(divLast);
					tr.appendChild(td);
					tbody.appendChild(tr);
				}
			}
		},
		
		/**
		 * Parse the path string to display the folders (plus file) in the breadcrumbs.
		 * 
		 * @param {Object} workspace
		 * @param {Object} component
		 * @param {Object} path
		 * @param {Object} isFile
		 */
		_parseAndDisplayPath: function(workspace, component, path, isFile, isHistoryBreadCrumb) {						
			//Get the path string as an array of items.
			var subElements = ScmUtil.parsePathIntoArray(path);
			var path = "";
			
			for(var i=0; i<subElements.length - 1; i++) {
				path = path + "/" + subElements[i];
				
				//If the path consists of 4 or more folders/files, only the first folder and the last three items
				//are displayed.
				if(subElements.length>4) {
					//First and last three items displayed.
					if(i==0 || i>subElements.length - 4) {
						var img = document.createElement("img");
						img.src = ScmUtil.getIconSrc("folder");
						img.className = "headerIcon";
						var anchor= document.createElement("a");
						anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, path));
						
//						anchor.title= subElements[i];
						var text= document.createTextNode(subElements[i]);
						anchor.appendChild(img);
						anchor.appendChild(text);
						this.currentPath.appendChild(anchor);
						
						var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
						this.currentPath.appendChild(text);
					}
					else if(i==1) {
						var text = document.createTextNode(this.messages.StreamWorkspaceView_56);
						this.currentPath.appendChild(text);
					}
				}
				else {
					var img = document.createElement("img");
					img.src = ScmUtil.getIconSrc("folder");
					img.className = "headerIcon";
					var anchor= document.createElement("a");
					anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, path));		
	
//					anchor.title= subElements[i];
					var text= document.createTextNode(subElements[i]);
					anchor.appendChild(img);
					anchor.appendChild(text);
					this.currentPath.appendChild(anchor);
					
					var text = document.createTextNode(this.messages.StreamWorkspaceView_25);
					this.currentPath.appendChild(text);
				}
			}
			//Display the last item (file or folder)
			
			if (isHistoryBreadCrumb) {
				var lastIndex = subElements.length - 1;
				
				var img = document.createElement("img");
				
				if (isFile) {
					img.src = ScmUtil.getIconSrc("file");
				}
				else {
					img.src = ScmUtil.getIconSrc("folder");
				}
				img.className = "headerIcon";
				path = path + "/" + subElements[lastIndex];
				var anchor= document.createElement("a");
				anchor.href= (ScmUtil.getBrowseElementURI(workspace.itemId, component.itemId, path));		

//				anchor.title= subElements[lastIndex];
				var text= document.createTextNode(subElements[lastIndex]);
				anchor.appendChild(img);
				anchor.appendChild(text);
				this.currentPath.appendChild(anchor);
			}
			else {
				var lastIndex = subElements.length - 1;
				
				var img = document.createElement("img");
				if (isFile) {
					img.src = ScmUtil.getIconSrc("file");
				}
				else {
					img.src = ScmUtil.getIconSrc("folder");
				}
				img.className = "headerIcon";
				this.currentPath.appendChild(img);
				var text = document.createTextNode(subElements[lastIndex]);
				this.currentPath.appendChild(text);
			}
		}
});

})();
