/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.common.TableTree"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("dojox.data.dom"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.Assert"); //$NON-NLS-1$

(function() {

var Assert					= com.ibm.team.apt.web.ui.internal.common.Assert;

var TAG_UNFILTERED_CHILDREN	= "unfilteredChildren"; //$NON-NLS-1$
var NODEID					= "nodeId"; //$NON-NLS-1$

dojo.declare("com.ibm.team.apt.web.ui.internal.view.common.TableTree", //$NON-NLS-1$
			 [dijit._Widget, dijit._Templated], {

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_table: null,
	_tableBody: null,
	_nodes: null,

	constructor: function() {
		this._rootNode= null;
		this._viewSorter= null;
		this._viewFilter= null;
	},

	// ---- api ------------------------------------------------------------------------------------------------------------

	setContent: function(rootNode) {
		this._rootNode= rootNode;
		this._rebuildTree();
	},

	getContent: function() {
		return this._rootNode;
	},

	setSorter: function(compareFn) {
		this._viewSorter= compareFn;
		this._rebuildTree();
	},
	
	setFilter: function(filterFn) {
		this._viewFilter= filterFn;
		this._rebuildTree();
	},
	
	refresh: function() {
		this._rebuildTree();
	},

	// ---- widget implementation ------------------------------------------------------------------------------------------

	templateString: "<table class=\"plannedItems\" dojoAttachPoint=\"_table\"></table>", //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$

	postCreate: function() {

	},
	
	destroy: function() {
		if (this._nodes != null) {
			this._removeChildren();
		}
	
		this._table= null;
		this._rootNode= null;
		this._viewSorter= null;
		this._viewFilter= null;

		this.inherited(arguments);
	},

	// ---- implementation -------------------------------------------------------------------------------------------------
	
	_removeChildren: function() {
		if (this._nodes != null) {
			for (var i = 0; i < this._nodes.length; i++) {
				this._nodes[i].destroy();
			}
			this._nodes= null;
		}

		dojox.data.dom.removeChildren(this._table);
	},
	
	_rebuildTree: function() {

		var expandedItems= {};
		dojo.query("tr.expanded", this._table).forEach(function(item) { //$NON-NLS-1$
			expandedItems[item.getAttribute(NODEID)]= true;
		});
		
		if (this._nodes != null) {
			this._removeChildren();
		}

		this._nodes= [];
		
		dojox.data.dom.removeChildren(this._table);
		this._tableBody= null;
		
		if (this._rootNode != null) {
			this._tableBody= document.createElement("tbody"); //$NON-NLS-1$
			this._setFilterState(this._rootNode);
			this._addNode(this._rootNode, null, expandedItems);
			this._table.appendChild(this._tableBody);
		}
	},

	_addNode: function(node, parentItem, expandedItems, insertAfter) {
		var treeItemWidget= node.getWidget(parentItem);
		
		var isExpanded= node.isRootNode() || expandedItems[node.getNodeId()] || false;
		
		if (!node.isRootNode() && treeItemWidget != null) {
			treeItemWidget.domNode.setAttribute(NODEID, node.getNodeId());
			
			if (insertAfter && insertAfter.nextSibling) {
				this._tableBody.insertBefore(treeItemWidget.domNode, insertAfter.nextSibling);
			} else {
				this._tableBody.appendChild(treeItemWidget.domNode);
			}

			Assert.isTrue(!node.isFiltered());
//			dojo.toggleClass(treeItemWidget.domNode, "filtered", node.isFiltered());
		}

		treeItemWidget.setExpansionState(isExpanded);
		
		dojo.toggleClass(treeItemWidget.domNode, "hidden", parentItem != null && !parentItem.isExpanded()); //$NON-NLS-1$

		var childNodes= node.getChildren();
		if (dojo.some(childNodes, function(node) { return !node.isFiltered() })) {
			if (this._viewSorter != null) {
				childNodes.sort(this._viewSorter);
			}

			if (isExpanded) {
				// while refreshing, we create all nodes directly
				for (var i= 0; i < childNodes.length; i++) {
					if (!childNodes[i].isFiltered())
						this._addNode(childNodes[i], treeItemWidget, expandedItems);
				}
			} else {
				var nodeCreator= dojo.hitch(this, function(nodesToCreate, yieldStartIndex, predecessorNode) {
					var yieldEndIndex= Math.min(nodesToCreate.length, yieldStartIndex + 10);
					for (var i= yieldStartIndex; i < yieldEndIndex; i++) {
						if (!nodesToCreate[i].isFiltered())
							predecessorNode= this._addNode(nodesToCreate[i], treeItemWidget, expandedItems, predecessorNode).domNode;
					}
	
					if (nodesToCreate.length > yieldEndIndex) {
						window.setTimeout(function() { nodeCreator(nodesToCreate, yieldEndIndex, predecessorNode) }, 0);
					}
				});

				treeItemWidget.setLazyChildCreator(dojo.hitch(this, function() {
					nodeCreator(childNodes, 0, treeItemWidget.domNode);
				}));
			}
		}
		
		return treeItemWidget;
	},

	_setFilterState: function(node) {
		var isFiltered= this._viewFilter != null && this._viewFilter(node);

		var hasVisibleChildren= false;
		var childNodes= node.getChildren();
		for (var i= 0; i < childNodes.length; i++) {
			hasVisibleChildren|= this._setFilterState(childNodes[i]);
		}

		node.setFiltered(isFiltered && !hasVisibleChildren);
		if (isFiltered && hasVisibleChildren) {
			node.setTag(TAG_UNFILTERED_CHILDREN);
		} else {
			node.clearTag(TAG_UNFILTERED_CHILDREN);
		}

		return !isFiltered || hasVisibleChildren;
	},
	
	__sentinel: null // terminates this class definition
});

/**
 * This helper method creates the arguments object for the widget's constructor.
 * @return {Object} a widget private object to be passed in as a constructor argument
 */
com.ibm.team.apt.web.ui.internal.view.common.TableTree.constructorArgs= function() {
	return {};
};

})();
