/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("dojox.data.dom"); // dojo.dom.removeChildren
dojo.require("dojo.date");
dojo.require("dojo.fx");
dojo.require("dojo.i18n");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("net.jazz.ajax.toolbox.lang.internal.Lang");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");

dojo.require("com.ibm.team.workitem.web.client.util");
dojo.require("com.ibm.team.workitem.web.client.internal.ImageClient");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");

dojo.requireLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

(function() {

var checkNotNull 			= net.jazz.ajax.toolbox.lang.checkNotNull;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

var WorkItemUtil		 	= com.ibm.team.workitem.web.client.util;
var EditorUtil	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;
var ImageClient				= com.ibm.team.workitem.web.client.internal.ImageClient;
var WorkItemClient 			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var ImagePool 				= com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

var getAuthenticatedContributor	= com.ibm.team.repository.web.client.session.getAuthenticatedContributor;
var isGuest 					= com.ibm.team.repository.web.client.session.isGuest;

var COMPATIBLE_TYPES		= [ ["defect", "task", "enhancement"] ];

var MAX_MESSAGE_LENGTH = 500;

/**
 * Top-level work item editor; has a persistent header (defined in this module)
 * and a set of child widgets (mapped in this._childWidgets).
 */
/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.WorkItemEditorHeader",
	[dijit._Widget, dijit._Templated],
	{
		templatePath: dojo.moduleUrl("com.ibm.team.workitem.web", "ui/internal/view/editor/templates/WorkItemEditorHeader.html"),
		// templateCssPath: dojo.uri.dojoUri("../com.ibm.team.workitem.web/ui/internal/view/editor/templates/WorkItemEditor.css"),
		_lastSaveTime: null,
		errorHandlingMap: {
			ALERT: function(self, message) {
				alert(message.message + "\n\n" + message.details)
			},
			ADD_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating = true;
				self.isValidationg = false;
				self.parentController.notifyLinkManagerToAddDuplicateLink();
				self.parentController.showLinksTab();
			},
			SET_RESOLUTION_TO_DUPLICATE_ACTION: function(self) {
				self.isValidating = true;
				self.parentController.showLinksTab();
				self.isValidationg = false;
				alert(self.messages.msgResolutionSet);
			},
			REMOVE_DUPLICATE_OF_LINK_ACTION: function(self) {
				self.isValidating = true;
				self.parentController.showLinksTab();
				self.isValidationg = false;
				self.parentController.notifyLinkManagerToRemoveDuplicateLink();
			}
		},
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.workitem.web.ui.internal", "WorkItemEditorMessages");

			this.titleSubscribeMe = this.messages.titleSubscribeMe;
			this.titleFindPotentialDuplicates = this.messages.titleFindPotentialDuplicates;
			this.titleCancelWorkItem = this.messages.titleCancelWorkItem;
			this.titleRefreshWorkItem = this.messages.titleRefreshWorkItem;
			this.titleRemoveWorkItem = this.messages.titleRemoveWorkItem;
			this.buttonCancel = this.messages.buttonCancel;
			this.buttonSave = this.messages.buttonSave;
			this.progressMessage = this.messages.busyLoading;
			this.labelSummary = this.messages.labelSummary;			
		},
		// populate the template with data, connect events, and show initial
		// child widget
		postCreate: function() {
			checkNotNull(this.parentController, "this.parentController");
			checkNotNull(this.editorUtil, "this.editorUtil");

			// Defect 66867 Truncated summary (pl locale this.labelSummary = "Podsumowanie: ";)
			if (this.labelSummary.length > 12) {
				this.summaryLabelDIV.className = "SummaryLabelVeryLong";
			}
			else 
				if (this.labelSummary.length > 9) {
					this.summaryLabelDIV.className = "SummaryLabelLong";
				}
			
			this._isInitialized = false;
			this._updateTypeIcon();
			
			this.PROPERTIES= {
				summary:	{requiredMarker: this.summaryRequiredMarker},
				state:		{requiredMarker: this.stateRequiredMarker},
				resolution:	{requiredMarker: this.resolutionRequiredMarker}
			};
		},
		markBusy: function(text) {
			if(!text) text = this.messages.busyWorking;
			// show the loading indicator
			this._showTypeAndId(text, "", true);				

		},
		focus: function() {
			if(this.summary) this.summary.focus();
		},
		reset: function(isTypeChange) {			
			// unbind work item data (this.typeName is cleared by this.markBusy)
			if (this.parentController.pageController.isGuest()) {
				dojo.addClass(this.summary, "GuestField");
				dojo.addClass(this.state, "GuestField");
				dojo.addClass(this.resolution, "GuestField");
				this.subscribeSelfCmd.style.display= "none";
			}
			
			dojox.data.dom.removeChildren(this.id);
			this.id.isDirty = false;
			this.findDuplicatesCmd.removeAttribute("href");
			dojo.removeClass(this.findDuplicatesCmd,"findDuplicatesIcon");
			dojo.addClass(this.findDuplicatesCmd,"findDuplicatesDisabledIcon");
//			if(!isTypeChange && (this._isInitialized || !this._workItem.id < 0 || this.summary.value == "")) {
//				if(!(this._workItem && this._workItem.attributes && this._workItem.attributes.summary 
//				   && this._workItem.attributes.summary.content &&
//				  		this._workItem.attributes.summary.content == this.summary.value)){
//							this.editorUtil._resetTextInput(this.summary);
//						}
//
//			}
			this.editorUtil._resetSelect(this.state);
			this.editorUtil._resetSelect(this.resolution);

			this._hasStickyMessage = false;
			this.isValidating = false;
			this._isAutomaticallySettingResolutionToDuplicate = false;
			this.clearValidationMessage();
			for (var id in this.PROPERTIES) {
				this.editorUtil._clearRequired(this.PROPERTIES[id].requiredMarker);
			}
			
			// reset UI state
			this.resolution.disabled= true;
			this.resolutionIcon.style.visibility= "visible";
			this.disableSave(/*isDisabled*/true);
			// reset internal state
			this.summary.__jazzChanged= false;
			
			if(isTypeChange) {
				this.summary.focus();
			}
		},

		setHeaderToolbarState: function(isNew) {
			if(isNew) {
				this.cancelCmd.style.display= "";
				this.subscribeSelfCmd.style.display= "none";
				this.refreshCmd.style.display= "none";
			} else {
				this.cancelCmd.style.display= "none";
				this.subscribeSelfCmd.style.display= "";
				this.refreshCmd.style.display= "";
			}
			
		},
		_showTypeAndId: function(typeName, idLabel, isLoading) {
				
			    if (isLoading) {
					dojox.data.dom.removeChildren(this.progressMsg);
					this.progressMsg.appendChild(document.createTextNode(typeName));
					this.typeIcon.style.visibility = "hidden";
					this.typeNameNode.style.visibility = "hidden";
					this.progressNode.style.visibility = "visible";
				}
				else {
					dojo.removeClass(this.typeIcon, "LoadingIcon");
					dojox.data.dom.removeChildren(this.id);
					dojox.data.dom.removeChildren(this.typeName);
					this.typeName.appendChild(document.createTextNode(typeName));
					if (idLabel && this.id.isDirty) {
						idLabel = idLabel.replace(/ \*/, "") + " *";
					}
					this.id.appendChild(document.createTextNode(idLabel));
					
					this.typeIcon.style.visibility = "visible";
					this.typeNameNode.style.visibility = "visible";
					this.progressNode.style.visibility = "hidden";
				}

		},		

		// bind work item data to the editor header (each sub-tab is responsible
		// for binding its own data)
		bindWorkItem: function(workItem, retainedState, isTypeChange, newTypeId) {
			// save a reference to the work item
			this._workItem= workItem;
			this._hasStickyMessage = false;
			this.isValidating = false;
			this._isAutomaticallySettingResolutionToDuplicate = false;
			this.clearValidationMessage();
			
			if(!isTypeChange ){
				this._updatedTypeName = null;
				this._updatedTypeIconSrc = null;
			}
			
			// enable or disable controls based on work item state
			this.updateSubscribeSelf();

			this._disableRefresh(false); // gets enabled as "reset" for new work items			
			this.state.disabled= this.parentController.isNewWorkItem()
			
			this.showWorkItemIdAndType(isTypeChange);
			this.editorUtil._bindLabelProviderToSelect(this._workItem.attributes.internalState, this.state, this.stateIcon);				
			this.editorUtil._bindLabelProviderToSelect(this._workItem.attributes.internalResolution, this.resolution, this.resolutionIcon);
			
			var isCompatibleTypeChange = true;	
			this.summary = this.summary ? this.summary : {};						
			if(isTypeChange){
				var isCompatibleTypeChange = false;
				if (retainedState.summary) {
					this.summary.__jazzChanged = true;
					this.summary.value = retainedState.summary
				} else {
					this.summary.value = EditorUtil.restoreTags(this._workItem.attributes.summary.content);
				}
				
				/*
				 * If it is a type change, the actions and resolutions
				 * can potentially be no longer valid.
				 * 
				 * We will assume that changes among defect,task and enhancement 
				 * are compatible and leave the actions and resolution enabled,
				 * but for changes away from or into these, we will disable
				 * the actions and resolution.
				 */
				var priorType = this._workItem.attributes.workItemType.id;
		

				for(var i=0;i<COMPATIBLE_TYPES.length;i++) {
					var typeSet = COMPATIBLE_TYPES[i];
					var found = false;
					for(var j=0;j<typeSet.length;j++) {
						if(typeSet[j]==priorType){
							found = true;
							break;
						}
					}
					if(found) {
						for(var j=0;j<typeSet.length;j++) {
							if(typeSet[j]==newTypeId){
								isCompatibleTypeChange = true;
							}
						}
					}
					if(isCompatibleTypeChange) break;					
				};
				if(isCompatibleTypeChange) {
					if(retainedState.state) {
						this.state.value =retainedState.state;
					}
	
					if(retainedState.resolution) {
						this.resolution.value=retainedState.resolution;
					}
				} else {
					this.state.disabled=true;
					this.resolution.disabled=true;
				}
				
			} else {
				if (this._isInitialized || !this._workItem.id < 0 || this.summary.value == "") {
					this.summary.value = EditorUtil.restoreTags(this._workItem.attributes.summary.content);
				} else {
					if(this._workItem.id < 0 && this.summary.value != ""){
						this.markDirty();
					}
				}
			}
			this._isInitialized = true;

			return isCompatibleTypeChange;
		},
		endOperation: function() {
			this.showWorkItemIdAndType();
		},
		showWorkItemIdAndType: function(isTypeChange) {
			// bind work item data to UI controls
			var typeName = this._workItem.attributes.workItemType.label;
			if(isTypeChange && this._updatedTypeName) {
				typeName = this._updatedTypeName;
			}
			var idLabel;
			if(this.parentController.isNewWorkItem()) {
				idLabel = EditorUtil.getNewWorkItemTimestampLabel(this._workItem);
			} else {
				idLabel = this._workItem.id;	
			}

			this._showTypeAndId(typeName, idLabel, /* isLoading */ false);			
	},
		
		bindOptionalValues: function(valMap, retainedState, isTypeChange, isCompatibleTypeChange) {
			
			var isCompatible = !(isTypeChange && !isCompatibleTypeChange);
			if (!this.parentController.isNewWorkItem()) {
				var stateAndActions 
				stateAndActions= [this._workItem.attributes.internalState];
				if(!isCompatible) {
					var fakeItem1= {
						iconUrl: "#", 
						id: "1",
						itemType: "smallString",
						label: this.messages.optionSaveToEnable
					}
					stateAndActions=[ fakeItem1 ];
					var fakeItem2= {
						iconUrl: "#", 
						id: "1",
						itemType: "smallString",
						label: ""
					}
					valMap.internalResolution=[ fakeItem2 ];
				}

				dojo.forEach(valMap.actionValues,
					function(actionValue) {
						if(actionValue.label == null && actionValue.id) {
							actionValue.label = actionValue.id;
						}
						stateAndActions.push(actionValue);
					});
				this.editorUtil._bindValuesToSelect(stateAndActions, this.state, this.stateIcon);
				if(isCompatible && isTypeChange && retainedState.state) {
							this.state.value = retainedState.state;
							this._updateResolutions(isTypeChange, retainedState);
						}
				
				
				if (valMap.internalResolution) {
						this.editorUtil._bindValuesToSelect(valMap.internalResolution, this.resolution, this.resolutionIcon);
						this.resolution.disabled= false;
						if(isTypeChange && retainedState.resolution) {
							this.resolution.value = retainedState.resolution;
						}	
				}
			}
			if(!isCompatible) {
					this.resolution.disabled=true;
					this.state.disabled=true;
			}
		
		},
		
		bindRequiredProperties: function(requiredProps) {
			for (var i=0; i<requiredProps.length; i++) {
				var requiredProp= this.PROPERTIES[requiredProps[i]];
				if (requiredProp) {
					this.editorUtil._markRequired(requiredProp.requiredMarker);
				}
			}
		},
		
		markFullyLoaded: function() {
			
			// If the type was updated, use that icon instead of the work item's
			if(this._updatedTypeIconSrc) {
				this._updateTypeIcon(this._updatedTypeIconSrc);
				return;
			}
			if (this._workItem.attributes.workItemType.iconUrl) {
				this._updateTypeIcon(ImageClient.getImageUrl(this._workItem.attributes.workItemType.iconUrl));
			} else {
				this._updateTypeIcon(/* hide it */ null);
			}
			this._enableFindDuplicatesIcon();
		},
		
		//
		// 'public' methods available to the controller
		//
		
		disableSave: function(isDisabled) {
			this.saveCmd.disabled= isDisabled;
			this.parentController.disableSave(isDisabled);
			this._enableFindDuplicatesIcon();
		},
				
		// implementation method for only the editor's form elements
		getFormElements: function() {
			var self= this;
			// include writable elements
			var formElements= {};
			
			// only include the 'summary' field if it's modified to avoid
			// unintentionally removing HTML markup from the Eclipse editor
			if (this.summary.__jazzChanged) {
				formElements.summary= this.summary.value;
			}
			
			formElements.internalState = this.state.value;
			formElements.internalResolution = this.resolution.value; 

			// and important read-only elements
//			formElements.itemId = this._workItem.itemId;
//			formElements.id = this._workItem.id;
					
			// The stateId is used to finde the project area, which cannot change
			// so get it from the work item
			if (!this.parentController.isNewWorkItem()) {
				formElements.stateId= this._workItem.stateId;
			}
					
			return formElements;
		},
		
		markDirty: function() {
			// Add the isDirty asterisk if it is not already there 
			if(this.id.innerHTML && this.id.innerHTML.indexOf("*") == -1) {
				this.id.isDirty = true;
				this.id.innerHTML= this.id.innerHTML + " *"; /* innerHTML OK */
			}
			// Enable top and bottom save button
			this.disableSave(/*isDisabled*/false);

		},
		
		/**
		 * Updates state of the "Subscribe Me" control to either add, remove or disabled
		 * based on what is currently allowed.
		 */
		updateSubscribeSelf: function() {
			if(!this._workItem) return;
			var loggedInContributor= getAuthenticatedContributor();
			var creatorId = -1;
			try {
				creatorId =  this._workItem.attributes.creator.itemId;
			}
			catch(e) {} // ignore;
			var isEnabled= !isGuest() && !this.parentController.isNewWorkItem() && loggedInContributor.itemId !== creatorId;
			if (isEnabled) {
				var isSubscriber= this.parentController.isSubscriber(loggedInContributor);
				dojo.removeClass(this.subscribeSelfCmd,"disabledIcon");
				this.subscribeSelfCmd.href= "#";
			}
			else {
				dojo.addClass(this.subscribeSelfCmd,"disabledIcon");
				this.subscribeSelfCmd.removeAttribute("href");
			}
			if (isEnabled && isSubscriber) {
				dojo.removeClass(this.subscribeSelfCmd,"subscribeIconAdd");
				dojo.addClass(this.subscribeSelfCmd,"subscribeIconRemove");
				this.subscribeSelfCmd.title= this.messages.titleUnsubscribeMe;
			}
			else {
				dojo.removeClass(this.subscribeSelfCmd,"subscribeIconRemove");
				dojo.addClass(this.subscribeSelfCmd,"subscribeIconAdd");
				this.subscribeSelfCmd.title= this.messages.titleSubscribeMe;
			}
		},

		// update the type (e.g. defect or task) based on the type code (type change)
		updateType: function(name, imgSrc, isInitialization) {
			if(isInitialization && this._updatedTypeName) return;
			imgSrc = imgSrc ||ImagePool.PIXEL.uri;
			if(!isInitialization) {
				this._updatedTypeName = name;
				this._updatedTypeIconSrc = imgSrc;
			}
			this.markBusy("Loading...");
//			this._showTypeAndId(name, "", false );
//			this._updateTypeIcon(ImageClient.getImageUrl(imgSrc ));
		},
		
		_updateTypeIcon: function(imgSrc) {
			if(dojo.hasClass(this.typeIcon, "validationMessageErrorIcon")){
				this.typeIcon.style.visibility="visible"
				return;
			}

			if(!imgSrc) {
				this.typeIcon.style.visibility="hidden"
			}
			
			this.typeIconSrc = imgSrc;
			this.typeIcon.src = imgSrc;
			if(imgSrc) {
				this.typeIcon.style.visibility="visible"
			}
		},					
		
		//
		// ui initialization methods
		//
		
		_disableRefresh: function(isDisabled) {
			if (isDisabled)
				dojo.addClass(this.refreshCmd,"disabledIcon");
			else
				dojo.removeClass(this.refreshCmd,"disabledIcon");
		},
		
		//
		// event handlers
		//
		
		_handleSummaryChange: function(e) {
			if(this.summary && this._workItem && this._workItem.attributes 
				&& this.summary.value == this._workItem.attributes.summary) return; // not changed
			this.parentController.conditionalMarkDirty(e);
		},

		_enableFindDuplicatesIcon: function() {
			if(!this._workItem) return;
			if (dojo.hasClass(this.findDuplicatesCmd,"findDuplicatesIcon")) return; // already enabled;
			var isEnabled = this._workItem.id > 0;
			if (!isEnabled) {
				if (this.summary && this.summary.value.length > 0) {
					isEnabled = true;
				}
				else {
					var descWidget = this.parentController.findObjectWithProperty({
						property: "isDescriptionManager"
					});
					if (descWidget && descWidget.getDescriptionValue) {
						if (descWidget.getDescriptionValue().length > 0) {
							isEnabled = true;
						}
					}
				}
			}
			if(isEnabled) {
				dojo.removeClass(this.findDuplicatesCmd,"findDuplicatesDisabledIcon");
				dojo.addClass(this.findDuplicatesCmd,"findDuplicatesIcon");
				this.findDuplicatesCmd.href= "#";	
			} else {
				this.findDuplicatesCmd.removeAttribute("href");
				dojo.removeClass(this.findDuplicatesCmd,"findDuplicatesIcon");
				dojo.addClass(this.findDuplicatesCmd,"findDuplicatesDisabledIcon");
			}
		},
		_handleFindDuplicatesClick: function(e) {
			e.preventDefault();
			if (dojo.hasClass(this.findDuplicatesCmd,"findDuplicatesDisabledIcon")) return;
			if(!this._workItem) return;
			if(this._workItem.id < 0) { // new
				var summary = dojo.trim(this.summary.value);
				var content = ""; 
				
				var descWidget = this.parentController.findObjectWithProperty({
					property: "isDescriptionManager"
				});
				if(descWidget && descWidget.getDescriptionValue) {
					content=dojo.trim(descWidget.getDescriptionValue());
				}
				this.parentController.pageController.openSimilarityQuery( {
					summary: summary,
					content: content
				});			
				
			} else {
				this.parentController.pageController.openSimilarityQuery( {
					itemId:  this._workItem.itemId,
					forceRefresh: true
				});
			}
			
						
		},
		_handleSubscribeSelfClick: function(e) {
			e.preventDefault();
			if (dojo.hasClass(this.subscribeSelfCmd,"disabledIcon")) return;
			this.parentController.markDirty();
			if (dojo.hasClass(this.subscribeSelfCmd, "subscribeIconAdd")) {
				this.parentController.addSubscribers([getAuthenticatedContributor()]);
			}
			else {
				this.parentController.deleteSubscribers([getAuthenticatedContributor()]);
			}
		},
		
		/*
		 * Respond to the user clicking the 'Save' control.
		 */
		_handleSaveClick: function(e) {
			this._lastSaveTime = new Date();
			this.parentController.save();
		},
		
		/*
		 * Respond to the user clicking the 'Refresh' control.
		 */
		_handleRefreshClick: function(e) {
			e.preventDefault();
			if (dojo.hasClass(this.refreshCmd,"disabledIcon")) {
				return;
			}
			// since we're going to update the work item
			this._lastSaveTime = null;
			this.parentController.refresh();
		},
		
		/*
		 * Respond to the user clicking the 'Remove' control.
		 */
		_handleRemoveClick: function(e) {
			e.preventDefault();
			if (dojo.hasClass(this.removeCmd,"disabledIcon")) {
				return;
			}
			// since we're going to update the work item
			if(!this.saveCmd.disabled) {
				if(!confirm(this.messages.promptCancelWorkItemEdit)) {
					return;
				}
			}
			
			this.parentController.pageController.removeEditor(this._workItem.id, true);
		},
		
		/*
		 * Respond to the user selecting a different value from the 'State'
		 * control by marking the form dirty and updating the 'Resolution'
		 * control.
		 */
		_handleStateChange: function(e) {
			if(e) {
				this._hasStickyMessage = false;
				this._isAutomaticallySettingResolutionToDuplicate = false;
			}
			this.parentController.markDirty();
			this.editorUtil._updateSelectImage(this.state);
			this._updateResolutions();
		},
		
		/*
		 * Update the set of values bound to the 'Resolution' control, based on
		 * the work item bound to the editor and the current value of the
		 * 'State' control.
		 */
		_updateResolutions: function(isTypeChange, retainedState) {
			var self= this;
			var serviceObj= {
				success: function(values) {
					if (values) {
						self.editorUtil._bindValuesToSelect(values, self.resolution, self.resolutionIcon);
						self.editorUtil._updateSelectImage(self.resolution);
						self.resolutionIcon.style.visibility= "visible";
						self.resolution.disabled= false;
						if(isTypeChange && retainedState.resolution) {
							self.resolution.value = retainedState.resolution;
						}
					} else {
						self.resolution.disabled = true;
						dojox.data.dom.removeChildren(self.resolution);
						self.editorUtil._addOption(self.resolution, new Option("", 0));
						self.resolutionIcon.style.visibility= "hidden";
					}
					self.updateValidationMessage(/* wasResolutionChanged */ !self._isAutomaticallySettingResolutionToDuplicate);
				},
				failure: function(errorObj) {
					self.parentController.handleError(errorObj, "A problem occurred while trying to fetch resolutions.");
				}
			};
			var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
			if (this.state.value === this._workItem.attributes.internalState.id) {
				WorkItemClient.getResolutions(srh, {itemId: this._workItem.itemId, status: this.state.value});
			} else {
				WorkItemClient.getResolutions(srh, {itemId: this._workItem.itemId, actionId: this.state.value});
			}
		},
		clearValidationMessage: function(force) {
			if(!force && this._skipNextClear ) {
				this._skipNextClear = false;
				return;
			} 
			this._skipNextClear = false; // forced
			if(!this._hasStickyMessage) {
				this.validationMessage.__jazzMessage = -1;
				dojox.data.dom.removeChildren(this.validationMessage);
			} 
			if(this.typeIconSrc) {
				this.typeIcon.src = this.typeIconSrc;
			}
			dojo.removeClass(this.typeIcon, "validationMessageErrorIcon");
			this.validationMessageSpan.className = "validationMessageEmpty";
		
		},
		hasDuplicateLink: function() {
			 return this.parentController.hasDuplicateOfLink();
		},
		setHeaderMessage: function(/* message, action, isSticky, isError */ message) {
			this.clearValidationMessage();
			if(!message) {
				return;
			}
			
			dojo.addClass(this.typeIcon, "validationMessageErrorIcon");
			
			if(!message.message || message.message.length == 0) {
				message.message = this.messages.errorUnhelpful;
			} else {
				if(message.message.lastIndexOf(".") != (message.message.length-1)) {
					message.message += ".";
				}
			}
            
            if (!message.details) {
                message.details = "";
            }
            else {
                if (message.details.lastIndexOf(".") != (message.details.length - 1)) {
                    message.details += ".";
                }
            }
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            }
            else {
                messageText = message.message + " " + message.details;
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this._hasStickyMessage = message.isSticky == true;
            this.validationMessage.__jazzMessage = message;
            if (message.isError) {
				this.typeIcon.src = ImagePool.ERROR;
                this.validationMessageSpan.className = "validationMessageError";
            }
            else {
				this.typeIcon.src = ImagePool.WARNING;
                this.validationMessageSpan.className = "validationMessageWarning";
				this._skipNextClear = true; // hack... we know there is an extra clear on setting resolutions
            }
            this.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.validationMessage, document.createTextNode(messageText));
			this.focus();
			window.scrollTo(0,0); // show message
		},
		updateValidationMessage: function(wasResolutionChanged) {
			if(this.isValidating) return;
			this.isValidating = true;
			// Check resolutions
			this.clearValidationMessage();
			if(!this.resolution || !this.resolution.options) return;
			var hasDuplicateLink = this.hasDuplicateLink();
			
//			var resolutionLabel = this.resolution.options[this.resolution.selectedIndex].__jazzLabel;
			var resolutionLabel = this.resolution.options[this.resolution.selectedIndex].text;
			if(resolutionLabel && resolutionLabel.toLowerCase() == "duplicate") {
				if(!hasDuplicateLink) {
					this.setHeaderMessage({
						message: this.messages.msgAddDuplicateOfLink,
						action: "ADD_DUPLICATE_OF_LINK_ACTION",
						details: this.messages.msgAddDuplicateOfLinkDetails,
						isSticky: false,
						isError: true
					})
				}
				
			} else {
				if(hasDuplicateLink) {
					if (wasResolutionChanged) {
						if (this._isAutomaticallySettingResolutionToDuplicate) {
							this._isAutomaticallySettingResolutionToDuplicate = false;
						}
						else {
							this.setHeaderMessage({
								message: this.messages.msgRemoveDuplicateOfLink,
								details: this.messages.msgRemoveDuplicateOfLinkDetails,
								action: "REMOVE_DUPLICATE_OF_LINK_ACTION",
								isSticky: false,
								isError: true
							})
							this.isValidating = false;
							return;
						}
					} else {
						this._isAutomaticallySettingResolutionToDuplicate = true;
						this._setResolutionToDuplicate(wasResolutionChanged);
					}
				}
			}
			this.isValidating = false;
			
		},
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if(!e || !e.target || !e.target.__jazzMessage) return;

			var actionFunc = this.errorHandlingMap[e.target.__jazzMessage.action];
			
			if(!actionFunc ) return;
			
			actionFunc(this,e.target.__jazzMessage );
 
		},
		_setResolutionToDuplicate: function(wasResolutionChanged) {
			if(!this.state || !this.state.options || !this.resolution || !this.resolution.options) return;
			
			var found = false;
			for(var i=0;i<this.state.options.length;i++ ) {
//				if(this.state.options[i].__jazzLabel =="Resolve" || this.state.options[i].__jazzLabel =="Resolved") {
				if(this.state.options[i].text =="Resolve" || this.state.options[i].text =="Resolved") {
					if(this.state.selectedIndex != i) this.state.selectedIndex =i; 
					this._handleStateChange(); 
					found = true;
					break;
				}
			}
			if (!found) {
						this.setHeaderMessage({
								message: this.messages.msgRemoveDuplicateOfLink,
								details: this.messages.msgRemoveDuplicateOfLinkDetails,
								action: "REMOVE_DUPLICATE_OF_LINK_ACTION",
								isSticky: false,
								isError: true
							})
				return;
			}
			found=false;
			var resolutionLabel;
			for(var i=0;i<this.resolution.options.length;i++ ) {
//				resolutionLabel = this.resolution.options[i].__jazzLabel;
				resolutionLabel = this.resolution.options[i].text;
				if(resolutionLabel && resolutionLabel.toLowerCase()=="duplicate") {
					this.resolution.selectedIndex =i;
					this._handleResolutionChange();
					found=true;
					break;
				}
			}
			if(!found) return;			
			this.updateValidationMessage();
			this.setHeaderMessage({
					message: this.messages.msgResolutionChangedToDuplicate,
					details: this.messages.msgResolutionChangedToDuplicateDetails,			
					action: "SET_RESOLUTION_TO_DUPLICATE_ACTION",
					isSticky: true,
					isError: false
			})

		},
		
		/*
		 * Respond to the user selecting a different value from the 'Resolution'
		 * control by marking the form dirty and updating the resolution icon. 
		 */
		_handleResolutionChange: function(e) {
			if(e) {
				this._isAutomaticallySettingResolutionToDuplicate = false;
				this._hasStickyMessage = false;
			}
			this.parentController.markDirty();
			this.editorUtil._updateSelectImage(this.resolution);
			this.updateValidationMessage(/*resolution changed*/ e!=null);
		},
		/*
		 * Update the last saved or loaded timestamp
		 */
		updateRefreshTimestamp: function(isNew) {
			if(isNew) {
				this.refreshCmd.title= this.messages.titleResetWorkItem;
				return;
			} else {
				this.refreshCmd.title= this.messages.titleRefreshWorkItem;
			}
					
			var now, prefix;
			if(this._lastSaveTime) {
				prefix = this.messages.labelSaved;
				now = this._lastSaveTime;
			} else {
				prefix = this.messages.labelLoaded;
				now = new Date();
			}
			try{
				var timeString = this.editorUtil._getDateString(now,true);
				var message;
				message = dojo.string.substitute(prefix, [ timeString ]);
				
				dojox.data.dom.replaceChildren(this.lastRefreshText, document.createTextNode(message));

				this.lastRefreshText.style.display="block";

			} catch(e) {
				// if there is a problem, just use the default now date
				this.lastRefreshText.style.display="none";
				console.debug(dojo.string.substitute(this.messages.errorInUpdateRefreshTimestamp, [ (""+e) , message ]));
			}
		}
	}
);

})();
