/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.editor.parts.CheckedListAttributePart");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.workitem.web.client.internal.WorkItemClient");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil");

(function() {

var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WorkItemClient			= com.ibm.team.workitem.web.client.internal.WorkItemClient;
var WidgetUtil  = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var EditorUtil 	= com.ibm.team.workitem.web.ui.internal.view.editor.EditorUtil;

dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.editor.parts.CheckedListAttributePart",
	[dijit._Widget, dijit._Templated],
	{
		checkBoxes: new Array(), // An array of checkboxes
		templateString: '<div class="EnumerationWidget NarrowWidget">' +
						'<img dojoAttachPoint="_image" class="PreFieldImage" />' +
						'<div class="fieldWrapper">' +
						'<div dojoAttachPoint="_checklist" >'  +
						'</div>' +
						'</div>' +
						'</div>',

		postCreate: function() {
			WidgetUtil.setSpacerImage(this._image);

			this._modifiers = this._modifiers ? this._modifiers : {};
			this.value = this.values[0].content;

			// If we have an associated enumeration valueset, attempt to
			// get it and fill in options later.
			this._fetchEnumeration({'enumerationId': this._getBoundEnumeration()});
				
			if (this._modifiers && this._modifiers.onCreate) {
				this.parentWidget[this._modifiers.onCreate](this, this.value);
			}
			if (this._modifiers && this._modifiers.onValueChanged) {
				this.parentWidget[this._modifiers.onValueChanged](this , this.value, /* isInitialization */ true);
			}
			if (this.parentController.pageController.isGuest()) {
				dojo.addClass(this._checklist, "GuestField");
			}
			this.isChanged = false;
		},
		
		_getBoundEnumeration: function() {
			var props = this.presentation.properties;
			var enumName = null;
			var i = 0;
			for (; props && i < props.length; i++) {
				if (props[i].key == "enumeration") {
					enumName = props[i].value;
					break;
				}
			}
			return enumName;
		},

		_createCheckBox: function(rootNode, enumLit, name) {
            var div = dojo.doc.createElement("div");
			rootNode.appendChild(div);
			var cbId = "rtccb"+name+enumLit.id;  

			var label = dojo.doc.createElement("label");
			label.style.verticalAlign = "middle";

			var txt = dojo.doc.createElement("span");
			txt.innerHTML = (enumLit.label) ? enumLit.label : enumLit.id;
			txt.style.verticalAlign = "middle";

			if (enumLit.iconUrl) {
				var img = dojo.doc.createElement("img");
				img.src=enumLit.iconUrl;
				img.style.verticalAlign = "middle";
				label.appendChild(img);
			}
			// Make the image and text part of the label, so it's clickable
			// (icon then text) 
			label.appendChild(txt);

			var input = dojo.doc.createElement("input");
			input.style.verticalAlign = "middle";

			// Order matters, first the checkbox, then the label	
			div.appendChild(input);
			div.appendChild(label);
			var cb = new dijit.form.CheckBox({'value': enumLit.id, 'name': name},input);
			
			// Link the label to the checkbox
			label.htmlFor = cb.id;
		  	return cb;
		},
		_disposeCheckBoxes: function() {
		  dojo.forEach(this.checkBoxes, function(node, i) {delete node;});
		  this.checkBoxes = new Array();	
		},
		
		_bindValuesToCheckBoxes: function(possibleValues, checklist, image) {
			var attrName = this.attributeId;
			this._disposeCheckBoxes();
			dojox.data.dom.removeChildren(checklist);
			
			dojo.forEach(possibleValues, function(enumLit, i) {
				var cb = this._createCheckBox(checklist, enumLit, attrName);
				this.connect(cb, "onClick", dojo.hitch(this, this._handleChange));
				this.checkBoxes[this.checkBoxes.length] = cb;
			}, this);

			var vals = this._splitValues(this.value);

			dojo.forEach(vals, function(val) {
				var found = false;
				dojo.forEach(this.checkBoxes, function(cb) {
					if (cb.value === val) {
						cb.setChecked(true);
						found = true;
					}
				}, this);
				// If the value isn't in the list of possible values, add it.
				if (found === false) {
					var cb = this._createCheckBox(checklist, {id: val}, attrName);
					this.connect(cb, "onClick", dojo.hitch(this, this._handleChange));
					this.checkBoxes[this.checkBoxes.length] = cb;
					cb.setChecked(true);
				}
			}, this);
		},

		_handleChange: function(e) {
			this.isChanged=true;
			if(this.parentWidget && this.parentWidget._handleDirtyingEvent) {
				this.parentWidget._handleDirtyingEvent(e);
			}
			var cb= EditorUtil._getOriginalTarget(e);
			var selectedValue = this._joinValues(this._getCurrentSelections(cb));
			if (selectedValue != this.value) this.value = selectedValue;
			
			if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
				this.parentWidget[this._modifiers.onValueChanged](this , selectedValue, /* isInitialization */ false);
			}
		},
		getFormElementValues: function(formElements) {
			var value = this.getValue();
			if (this.isChanged) {
				formElements[this.attributeId] = value;
			}
		},
		setFormElementValues: function(retainedState){
			var oldValue = retainedState[this.attributeId]; 
			var selectValue = this.getValue();
		 	if (oldValue) {
				if(selectValue != oldValue) {
					selectValue = oldValue;
					this.isChanged = true;
					if(this._modifiers && this._modifiers.onValueChanged && this.parentWidget && this.parentWidget[this._modifiers.onValueChanged]) {
						this.parentWidget[this._modifiers.onValueChanged](this , selectValue);
					}
				}
			}
		 },
		 setValue: function(value) {
		 	// Assume that value is a single string with comma-separated values.
		 	if (dojo.isString(value)) {
				this.value = value;
			}
		 },
	     getValue: function() {
			return this.value;
		 },
		 _getCurrentSelections: function(checkboxDiv) {
			var vals = new Array(); 
			dojo.forEach(this.checkBoxes, function(checkbox) {
				if (checkbox.checked) {
					vals[vals.length] = checkbox.value;
				}
			});
			return vals;
		},
		setReadOnly: function(isReadOnly) {
			this._checklist.disabled=isReadOnly;
		},		
		_splitValues: function(vals) {
			if (dojo.isString(vals)) {
				if (vals === "") {
					return new Array();
				} else {
					return vals.split(',');
				}
			}
			return null;
		},
		_joinValues: function(arr) {
			if (dojo.isArray(arr)) {
				return arr.join(',');
			}
			return "";
		},
		_fetchEnumeration: function(params) {
			if (params.enumerationId) {
				var self= this;
				var serviceObj= {
					success: function(values) {
						self._possibleValues = values;
						self._bindValuesToCheckBoxes(values, self._checklist, self._image);
						//self.updateValidationMessage( !self._isAutomaticallySettingResolutionToDuplicate);
					},
					failure: function(errorObj) {
						self.parentController.handleError(errorObj, "A problem occurred while trying to fetch enumeration.");
					}
				};
				var srh= new ServiceResponseHandler(serviceObj, "success", "failure");
				WorkItemClient.getEnumerationValues(srh, {enumerationId: params.enumerationId});
			} else {
				this._possibleValues = new Array();
				this._bindValuesToCheckBoxes(this._possibleValues, this._checklist, this._image);
			}
		}
	}
);

})();
