/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget");


	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");

dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.layout.SectionListTableWidget");
dojo.require("com.ibm.team.workitem.web.ui.internal.view.common.ImagePool");

(function() {

var WidgetUtil = com.ibm.team.workitem.web.ui.internal.view.layout.WidgetUtil;
var ImagePool = com.ibm.team.workitem.web.ui.internal.view.common.ImagePool;

/* defineWidget */ dojo.declare(
	"com.ibm.team.workitem.web.ui.internal.view.layout.SectionListWidget",
	[dijit._Widget, dijit._Templated],
	{

		templateString: '<div class="SectionListWidget">' +
						'<div dojoAttachPoint="_tableNode"></div>' +
						'</div>',

		postCreate: function() {
			var defaultClass = "SectionListTableHeader";
			var columnClass = this.isFirst ? defaultClass : false;
					if(this.noPadding) {
						columnClass = "SectionListTableHeaderPlain";
					}
					
			var tableClass = this.tableClass ? this.tableClass : false;
 
			this._tableWidget = new com.ibm.team.workitem.web.ui.internal.view.layout.SectionListTableWidget(
				{ parentWidget: this, parentController: this.parentController,
					tableClass: this.isFirst ? defaultClass : this.tableClass,
					columnClass: columnClass
					} );
				 
			
			if(this.noBorder) {
				try{
					this._tableWidget._table.style.border = "0";
				} catch(e) {};
			}
		
			if(this.heading) {
				this.setHeading(this.heading);
			}
			
			if(this.rows) {
				this.setRows(this.rows);
			}
			
			this._tableNode.appendChild(this._tableWidget.domNode);
		},
		destroyWidget: function() {
			this.parentController.pageController.removeAllMouseListeners(this);
			
			this.destroy();
		},
		setHeading: function(heading){
			this.heading= heading;
			this.heading = this.heading ? this.heading :  {};
			this.heading.columnWidgets = this.heading.columnWidgets ? this.heading.columnWidgets :  [];

			this.renderHeading(heading.columnWidgets, heading.value, heading.deleteAction, heading.restoreAction);
		},
		setRows: function(rows) {
			this.rows = rows;
			this.rows = this.rows  ? this.rows  :  {};
			this.rows.itemList = this.rows.itemList ? this.rows.itemList :  [];
			
			for (var i = 0; i < this.rows.itemList.length; i++) {
				var isLast = i == this.rows.itemList.length -1;
				var item = this.rows.itemList[i];
				if(this.parentController.pageController.isGuest()) {
					item.showDelete = false;
				}
				if(item) {
					this.addItem(item.columnWidgets, item.value, item.showDelete != false ? this.rows.deleteAction : null,  item.showDelete != false ? this.rows.restoreAction : null, isLast );	
				} else {
					console.debug("Empty item in section list widget items [i=" + i + "].");
				}
			}
		},
		renderHeading: function(widgets, value, deleteAction, restoreAction) {

			var row = this._addRow(widget);
			if(this.isHeading) 	row.className = "SectionListTableHeadingRow"; // Column titles only heading
			for(var i=0;i<widgets.length; i++) {
				var widget = widgets[i];
				this._tableWidget.addColumnWidget(row, widget, widget.columnParms);

			}
			var isGuest = this.parentController.pageController.isGuest();
			if(widgets.length > 0) {
				var className = widget && widget.columnParms && widget.columnParms.className ? widget.columnParms.className : null;
				
				if(!isGuest && value && value._isDeleted && restoreAction) {
					dojo.addClass(row,  "cellDeleted");
					this._createActionColumn(row, restoreAction,  ImagePool.RESTORE, ImagePool.RESTORE_HOVER, className);
				}
				if(!isGuest && value && !value._isDeleted && deleteAction) {
					this._createActionColumn(row, deleteAction,  ImagePool.REMOVE, ImagePool.REMOVE_HOVER, className);
				}
				if(value) row._value = value;
				this.setRowBackground(row, false);
			}

		},
		_addRow: function (value, isLast) {
			var parms = {};
			if(isLast) {
				parms = {className: "SectionListTableRowLast" };
			}
			var row  = this._tableWidget.addRow(parms);
			row._mouseListener = this;
			row._mouseOverActions = [];
			row._mouseOutActions = [];
			row._value = value;
			this.parentController.pageController.addMouseListener(this, row,{});
//			dojo.connect(row, "onmouseover", this._handleMouseOverRow);
//			dojo.connect(row, "onmouseout", this._handleMouseOutRow);
			return row;
		},
		handlePageMouseOver: function (e, listener) {
			this._handleMouseOverRow(e, listener);
		},
		handlePageMouseOut: function (e, listener) {
			this._handleMouseOutRow(e, listener);
		},
		_createActionColumn: function (row, action, imgSrc, imgSrc_hover, className) {
			    if(!row || !imgSrc || !this.parentWidget) return;
				var img = document.createElement("IMG");
				img.src = imgSrc;
				img.onmouseover = function(e){
					img.src = imgSrc_hover;
				};
				img.onmouseout = function(e){
					img.src = imgSrc;
				}
				img.value = row._value;
				img.parentWidget = this.parentWidget;
				img.className = "SectionListActionWidget";
				row._actionImage = img;
				if(!action) {
					img.style.visibility = "hidden";
				}
				var cell = this._tableWidget.addColumn(row, img, action, className);
				dojo.addClass(cell,"SectionListActionColumn");
				if(action && this.parentWidget[action.selector]) {
					img.title = action.title;
					this.connect(cell, "onclick", this.parentWidget[action.selector]);
					row._mouseOverActions.push( { actionSelector: "_highlight", actionTarget: row });
					row._mouseOutActions.push( { actionSelector: "_unhighlight", actionTarget: row });
				}

				if( row._value &&  row._value._isDeleted) {
					dojo.addClass(row,  "cellDeleted");
				}
		},
		
		addItem: function (itemWidgets, value, deleteAction, restoreAction, isLast) {
			if(!itemWidgets) return;
			var row = this._addRow(value, isLast);

			for(var i=0;i<itemWidgets.length; i++) {
				var widget = itemWidgets[i];
				
				this._tableWidget.addColumnWidget(row, widget, widget.columnParms);

			}
			var className = widget && widget.columnParms && widget.columnParms.className ? widget.columnParms.className : null;

			if(value && value._isDeleted && restoreAction) {
				this._createActionColumn(row, restoreAction,  ImagePool.RESTORE, ImagePool.RESTORE_HOVER, className);
			}
			if(value && !value._isDeleted && deleteAction) {
				this._createActionColumn(row, deleteAction,  ImagePool.REMOVE, ImagePool.REMOVE_HOVER, className);
			}
			this.setRowBackground(row, false);
		},
		setRowBackground: function(row, isHovering) {
			if(!row._value) {
				return;
			}
			if(row._value.isNotHighlighted) {
				row.style.backgroundColor =  "";
				return;
			}
			if (row._value._isDeleted) {
					row.style.backgroundColor = isHovering ? "#FFF4E1" : "#FFF9EF";
					if(row._actionImage) {
							row._actionImage.style.visibility = "visible";
						}
				}
				else {
					if (row._value._isNew) {
						row.style.backgroundColor = isHovering ? "#EEF4F8" : "#F5F8FB";
						if(row._actionImage) {
							row._actionImage.style.visibility = "visible";
						}
					}
					else {
						row.style.backgroundColor = isHovering ? "#F6F6F6" :  "";
						if(row._actionImage) {
							row._actionImage.style.visibility = isHovering ?  "visible" : "hidden";
						}
					}
				}
		},
		getLabelWidget: function(layout) {
			var widget = this._getFakeLabelWidget(layout.label, layout.style, layout.href);
			widget.columnParms= layout;
					
			return widget;
			},
		_getFakeLabelWidget: function(label, style, href) {
			var span = document.createElement("SPAN");
			if(!href) {
				span.appendChild(document.createTextNode(label));
			} else {
				var anchor = document. createElement("A");
				anchor.href = href;
				anchor.appendChild(document.createTextNode(label));
				span.appendChild(anchor);
			}

			
			for(var attr in style ) { 
					span.style[attr] = style[attr];
				}
			return { domNode: span }
		},
		
		_highlight: function(row) {
			if(row.isHighlighted) return;
			row.isHighlighted = true;
			this.setRowBackground(row, true);
		},
		_unhighlight: function(row) {
			if(!row.isHighlighted) return;
			row.isHighlighted = false;
			this.setRowBackground(row, false);
		},
		_performActions: function(actions) {
			if(!actions) return;
			for(var i=0; i<actions.length; i++) {
				var action = actions[i];
				if(action.actionSelector) {
					this[action.actionSelector](action.actionTarget);					
				}
			}
		},
		_handleMouseOverRow: function(e, row) {
			var target = row.domNode;
			if(!target || !target._mouseListener || !target._mouseOverActions) return;
			target._mouseListener._performActions(target._mouseOverActions);
		},
		_handleMouseOutRow: function(e, row) {
			var target = row.domNode;
			if(!target || !target._mouseListener || !target._mouseOutActions) return;
			target._mouseListener._performActions(target._mouseOutActions);
		}
	}
);

})();
