/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.apt.web.ui.internal.view.navigation.KeywordQuery"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dijit.Menu"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.client.util"); //$NON-NLS-1$

dojo.require("com.ibm.team.apt.web.ui.internal.common.QueryDescriptor"); //$NON-NLS-1$

dojo.requireLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require("dojo.i18n"); //$NON-NLS-1$

(function() {

var PlanningUtil= com.ibm.team.apt.web.client.util;
var QueryDescriptor= com.ibm.team.apt.web.ui.internal.common.QueryDescriptor;

var bind= dojo.string.substitute;
var MSGS= dojo.i18n.getLocalization("com.ibm.team.apt.web.ui.internal", "IterationPlanMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Widget to display keyword search controls. Contains a text input field and a submit button. Triggers a search
 * of iteration plans based on the text submitted.
 */
dojo.declare("com.ibm.team.apt.web.ui.internal.view.navigation.KeywordQuery", //$NON-NLS-1$ 
			 [dijit._Widget, dijit._Templated], {

	CURRENT_PLANS_LABEL: MSGS.PREDEFQUERY_CURRENT,
	ALL_PLANS_LABEL: MSGS.PREDEFQUERY_ALL,
	DROP_DOWN_ICON_CLASS: "planIcon", //$NON-NLS-1$
	ACTIVE_SEARCH_ICON_CLASS: "search-button-active", //$NON-NLS-1$

	_tooltipLabel: MSGS.PREDEFQUERY_SEARCH_TOOLTIP_LABEL,

	// ---- constructor mixed-in properties --------------------------------------------------------------------------------	

	_searchForm: null,
	_queryType: null,
	_searchText: null,
	_searchButton: null,
	
	
	constructor: function() {
		this._currentOnly = false;
	},

	// ---- API ------------------------------------------------------------------------------------------------------------	

	clear: function() {
		this._searchText.value= ""; //$NON-NLS-1$
	},


	focus: function() {
		this._searchText.focus();
	},

	// ---- widget lifecycle implementation --------------------------------------------------------------------------------

	templatePath: dojo.moduleUrl("com.ibm.team.apt.web", "ui/internal/view/navigation/templates/KeywordQuery.html"), //$NON-NLS-1$ //$NON-NLS-2$
	
	postCreate: function() {

		this.connect(this._searchText, "onkeyup", this._checkEmpty); //$NON-NLS-1$
		this.connect(this._searchButton, "onclick", this._handleSearch); //$NON-NLS-1$
		this.connect(this._searchForm, "onsubmit", this._handleSearch); //$NON-NLS-1$
		this.connect(this._queryType,"onclick", this._openMenu); //$NON-NLS-1$
		this.connect(this._searchText,"onkeypress",this._sniffKeySubmit); //$NON-NLS-1$
		this.connect(this._queryType,"onkeypress",this._sniffKey); //$NON-NLS-1$
		
		this._queryType.title = MSGS.PREDEFQUERY_DROPDOWN_TOOLTIP_LABEL_CURRENT;
	},

	// ---- implementation -------------------------------------------------------------------------------------------------


	_checkEmpty: function() {
		if (this._searchText.value.length < 1) {
			dojo.removeClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
		} else {
			dojo.addClass(this._searchButton, this.ACTIVE_SEARCH_ICON_CLASS);
		}
	},

	_handleSearch: function(e) {
		if (e) {
			e.preventDefault();
		}
		var trimmedStr = dojo.trim(this._searchText.value);
		if (trimmedStr.length === 0) {
			return;
		} else {
			var queryDescriptor= new QueryDescriptor(this._searchText.value, this._currentOnly ? PlanningUtil.QUERY_CURRENTPLANS : null);
			window.location.href = PlanningUtil.getSearchURI(queryDescriptor);
		}
	},
	
	_sniffKey: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		if (e.keyCode == dojo.keys.DOWN_ARROW || e.keyCode == dojo.keys.ENTER) 
			this._openMenu();
	},
	
	_sniffKeySubmit: function(e){
		if (e.keyCode == dojo.keys.ENTER) 
			this._handleSearch();
	},
	
	_openMenu: function(e) {
		if (e) {
			dojo.stopEvent(e);
		}
		if (this._menu) {
			this._closeMenu();
			return;
		}
		if (this._currentOnly) {
			var allIcon = this.DROP_DOWN_ICON_CLASS;
			var allOpenIcon = this.DROP_DOWN_ICON_CLASS + "-selected"; //$NON-NLS-1$
			this._queryType.title = MSGS.PREDEFQUERY_DROPDOWN_TOOLTIP_LABEL_ALL;
		} else {
			var allIcon = this.DROP_DOWN_ICON_CLASS + "-selected"; //$NON-NLS-1$
			var allOpenIcon = this.DROP_DOWN_ICON_CLASS;
			this._queryType.title = MSGS.PREDEFQUERY_DROPDOWN_TOOLTIP_LABEL_CURRENT;
		}
		var menu = new dijit.Menu();
		menu.addChild(new dijit.MenuItem({
			label: this.ALL_PLANS_LABEL,
			iconClass: allIcon,
			onClick: dojo.hitch(this, this._searchAllPlans)
		}));
		menu.addChild(new dijit.MenuItem({
			label: this.CURRENT_PLANS_LABEL,
			iconClass: allOpenIcon,
			onClick: dojo.hitch(this, this._searchCurrentPlans)
		}));
		
		var pos = dojo.coords(this._searchForm);
		pos.x += document.documentElement.scrollLeft;
		pos.y += document.documentElement.scrollTop;
		
		dojo.addClass(this._queryType, "QueryType-selected"); //$NON-NLS-1$
		dijit.popup.open({
			popup: menu,
			parent: this,
			x: pos.x + 5,
			y: pos.y + pos.h - 2,
			onExecute: dojo.hitch(this, this._closeMenu),
			orient: 'BL' //$NON-NLS-1$
		});
		
		//register event handler that causes menu to close
		this._menuCloseOnBody = this.connect(this, "_onBlur", this._closeMenu); //$NON-NLS-1$
		
		this._menu = menu;
		
		menu.focus();
	},
	
	_closeMenu: function(e){
		if (e) {
			dojo.stopEvent(e);
		}
		dojo.removeClass(this._queryType, "QueryType-selected"); //$NON-NLS-1$
		this.disconnect(this._menuCloseOnBody);
		dijit.popup.close(this._menu);
		this._menu = null;
		this._searchText.focus();
	},
	
	_searchAllPlans: function() {
		this._currentOnly = false;
	},
	
	_searchCurrentPlans: function() {
		this._currentOnly = true;
	},

	__sentinel: null // terminates this class definition
});

})();